import re
from datetime import datetime

import bot


def test_build_obfuscated_filename_hides_source_name():
    fixed_now = datetime(2025, 12, 15, 10, 0, 0, 123000, tzinfo=bot.UTC)
    result = bot._build_obfuscated_filename(
        "photo_AQADTAtrGylIAVZ9.jpg",
        "image/jpeg",
        salt="salt",
        now=fixed_now,
        monotonic_ns=123456789,
    )

    assert result.endswith(".jpg")
    assert result.startswith("20251215_100000123")
    assert "photo_AQADTAtrGylIAVZ9" not in result
    assert re.fullmatch(r"20251215_100000123-[0-9a-f]{12}\.jpg", result)


def test_build_obfuscated_filename_sanitizes_extension():
    fixed_now = datetime(2025, 12, 15, 10, 0, 0, 0, tzinfo=bot.UTC)
    result = bot._build_obfuscated_filename(
        "weird name$$$.tar.gz",
        None,
        salt="salt",
        now=fixed_now,
        monotonic_ns=1,
    )

    assert result.startswith("20251215_100000000-")
    assert result.endswith(".gz")



"""命令管理相关的数据模型。"""
from __future__ import annotations

import sys
from dataclasses import dataclass, field
from typing import Optional, Tuple

from tasks.models import shanghai_now_iso, ensure_shanghai_iso

# Python 3.10 之前 dataclass 没有 slots 参数，动态传参兼容旧版本。
_DATACLASS_SLOT_KW = {"slots": True} if sys.version_info >= (3, 10) else {}


@dataclass(**_DATACLASS_SLOT_KW)
class CommandDefinition:
    """记录单条命令的元数据。"""

    id: int
    project_slug: str
    name: str
    title: str
    command: str
    scope: str = "project"
    description: str = ""
    timeout: int = 120
    enabled: bool = True
    created_at: str = field(default_factory=shanghai_now_iso)
    updated_at: str = field(default_factory=shanghai_now_iso)
    aliases: Tuple[str, ...] = ()


@dataclass(**_DATACLASS_SLOT_KW)
class CommandHistoryRecord:
    """描述命令执行的审计记录。"""

    id: int
    command_id: int
    project_slug: str
    command_name: str
    command_title: Optional[str]
    trigger: Optional[str]
    actor_id: Optional[int]
    actor_username: Optional[str]
    actor_name: Optional[str]
    exit_code: Optional[int]
    status: str
    output: Optional[str]
    error: Optional[str]
    started_at: str = field(default_factory=shanghai_now_iso)
    finished_at: str = field(default_factory=shanghai_now_iso)

    def ensure_timestamps(self) -> None:
        """规范化时间字符串，便于外部展示。"""

        self.started_at = ensure_shanghai_iso(self.started_at) or self.started_at
        self.finished_at = ensure_shanghai_iso(self.finished_at) or self.finished_at



#!/usr/bin/env python3
"""Master 启动后的健康检查脚本。

流程：
1. 等待 master 日志出现启动标记。
2. 调用 MasterManager 启动指定 worker（默认 hyphavibebotbackend）。
3. 自动发现该 worker 的 chat_id（优先 state 文件，其次读取最新日志）。
4. 通过 Telegram Bot API 向该 chat 发送探针消息，确认发送成功。
5. 任何步骤失败则抛出异常，并尝试通知管理员。

注意：本脚本不会自动重试重启，仅返回非零退出码供外层脚本处理。
"""
from __future__ import annotations

import argparse
import asyncio
import json
import os
import re
import sys
import time
from pathlib import Path
from typing import Optional
from urllib.error import URLError, HTTPError
from urllib.request import Request, urlopen

# 导入 master 中的配置与工具，复用项目解析逻辑
ROOT_DIR = Path(__file__).resolve().parent.parent
ROOT_DIR_STR = str(ROOT_DIR)
if ROOT_DIR_STR not in sys.path:
    # 确保可以从仓库根目录导入 master 模块
    sys.path.insert(0, ROOT_DIR_STR)

import master  # type: ignore
from project_repository import ProjectRepository
DEFAULT_MASTER_LOG = master.LOG_ROOT_PATH / "vibe.log"
DEFAULT_TIMEOUT_MASTER = 60.0
DEFAULT_TIMEOUT_PROBE = 15.0
PROBE_TEXT = "hello"
REPOSITORY = ProjectRepository(master.CONFIG_DB_PATH, master.CONFIG_PATH)


def _load_project(project_id: str) -> master.ProjectConfig:
    """根据 slug 或 bot 名获取项目配置，失败时列出可选项。"""

    record = REPOSITORY.get_by_slug(project_id)
    if record is None:
        record = REPOSITORY.get_by_bot_name(project_id)
    if record is None:
        available = [r.project_slug for r in REPOSITORY.list_projects()]
        raise RuntimeError(f"未找到项目 {project_id}，可选项目: {available}")
    return master.ProjectConfig.from_dict(record.to_dict())


def _wait_for_log_flag(path: Path, pattern: str, timeout: float) -> None:
    """在超时时间内等待日志出现特定标记。"""

    deadline = time.monotonic() + timeout
    position = 0
    while time.monotonic() < deadline:
        if path.exists():
            if position == 0:
                position = path.stat().st_size
            with path.open("r", encoding="utf-8", errors="ignore") as fh:
                fh.seek(position)
                while time.monotonic() < deadline:
                    line = fh.readline()
                    if not line:
                        time.sleep(0.5)
                        continue
                    position = fh.tell()
                    if pattern in line:
                        return
        time.sleep(0.5)
    raise TimeoutError(f"在 {timeout:.0f} 秒内未检测到日志标记: {pattern}")


def _extract_chat_id_from_logs(log_path: Path) -> Optional[int]:
    """从日志文件倒序查找最近的 chat_id。"""

    if not log_path.exists():
        return None
    pattern = re.compile(r"chat=(-?\d+)")
    try:
        lines = log_path.read_text(encoding="utf-8", errors="ignore").splitlines()
    except Exception:
        return None
    for line in reversed(lines[-200:]):  # 反向查找最近的记录
        match = pattern.search(line)
        if match:
            try:
                return int(match.group(1))
            except ValueError:
                continue
    return None


def _ensure_chat_id(cfg: master.ProjectConfig, manager: master.MasterManager) -> int:
    """确保任务分配有 chat_id，必要时从日志回填并写回 state。"""

    state = manager.state_store.data.get(cfg.project_slug)
    if state and state.chat_id:
        return int(state.chat_id)
    # 回落到日志查找
    log_dir = master.LOG_ROOT_PATH / (cfg.default_model.lower()) / cfg.project_slug
    chat_id = _extract_chat_id_from_logs(log_dir / "run_bot.log")
    if chat_id is None:
        raise RuntimeError(
            "无法自动获取 chat_id，请先手动与该 bot 发生一次对话以写入 state/log"
        )
    # 将发现的 chat_id 写回 state，便于下次复用
    manager.state_store.update(cfg.project_slug, chat_id=chat_id)
    return chat_id


def _send_probe(bot_token: str, chat_id: int, text: str, timeout: float) -> None:
    """向指定 chat 发送探针消息，验证 Telegram API 可用。"""

    url = f"https://api.telegram.org/bot{bot_token}/sendMessage"
    payload = json.dumps({"chat_id": chat_id, "text": text, "disable_notification": True}).encode("utf-8")
    request = Request(url, data=payload, headers={"Content-Type": "application/json"}, method="POST")
    try:
        with urlopen(request, timeout=timeout) as resp:
            data = json.loads(resp.read().decode("utf-8"))
    except HTTPError as exc:  # pragma: no cover - 网络异常时抛出
        raise RuntimeError(f"发送探针消息失败，HTTP {exc.code}: {exc.reason}") from exc
    except URLError as exc:  # pragma: no cover - 网络异常时抛出
        raise RuntimeError(f"发送探针消息失败: {exc}") from exc
    if not data.get("ok"):
        raise RuntimeError(f"发送探针消息失败: {data}")


def _format_admin_notice(reason: str) -> str:
    """生成通知管理员的告警文本。"""

    return (
        "Master 重启健康检查失败\n"
        f"原因：{reason}\n"
        "请尽快登录服务器排查（start.log / vibe.log）。"
    )


def _notify_admins(reason: str) -> None:
    """如果 master token 可用，则向管理员列表广播失败原因。"""

    master_token = os.environ.get("MASTER_BOT_TOKEN")
    if not master_token:
        return
    admins = master._collect_admin_targets()
    if not admins:
        return
    message = _format_admin_notice(reason)
    url = f"https://api.telegram.org/bot{master_token}/sendMessage"
    for chat_id in admins:
        payload = json.dumps(
            {"chat_id": chat_id, "text": message, "disable_notification": False}
        ).encode("utf-8")
        request = Request(url, data=payload, headers={"Content-Type": "application/json"}, method="POST")
        try:
            with urlopen(request, timeout=10):
                pass
        except Exception:
            continue


def _ensure_worker(cfg: master.ProjectConfig) -> master.MasterManager:
    """启动指定项目 worker，并返回临时构造的 MasterManager。"""

    records = REPOSITORY.list_projects()
    configs = [master.ProjectConfig.from_dict(record.to_dict()) for record in records]
    state_store = master.StateStore(
        master.STATE_PATH, {item.project_slug: item for item in configs}
    )
    manager = master.MasterManager(configs, state_store=state_store)

    async def _run() -> None:
        """协程执行实际的 stop/start 流程。"""
        # 确保先停止旧实例（若存在）
        try:
            await manager.stop_worker(cfg)
        except Exception:
            pass
        await manager.run_worker(cfg)

    asyncio.run(_run())
    return manager


def main() -> int:
    """命令行入口，执行 master 健康检查并返回退出码。"""

    parser = argparse.ArgumentParser(description="Master 启动后的健康检查")
    parser.add_argument("--project", default="hyphavibebotbackend", help="项目 slug 或 bot 名称")
    parser.add_argument("--master-log", default=str(DEFAULT_MASTER_LOG), help="master 日志路径")
    parser.add_argument("--master-timeout", type=float, default=DEFAULT_TIMEOUT_MASTER, help="master 日志等待超时时间 (秒)")
    parser.add_argument("--probe-timeout", type=float, default=DEFAULT_TIMEOUT_PROBE, help="Telegram 探针超时时间 (秒)")
    args = parser.parse_args()

    project_id = master._sanitize_slug(args.project)
    master_log = Path(args.master_log)

    try:
        _wait_for_log_flag(master_log, "Master 已启动，监听管理员指令。", args.master_timeout)
        cfg = _load_project(project_id)
        manager = _ensure_worker(cfg)
        chat_id = _ensure_chat_id(cfg, manager)
        _send_probe(cfg.bot_token, chat_id, PROBE_TEXT, args.probe_timeout)
    except Exception as exc:
        reason = str(exc)
        _notify_admins(reason)
        print(f"[healthcheck] 失败: {reason}", file=sys.stderr)
        return 1
    else:
        print(
            "[healthcheck] 成功: master 已就绪，"
            f"worker={cfg.display_name} 启动完成，chat_id={chat_id}，已发送探针消息"
        )
        return 0


if __name__ == "__main__":
    sys.exit(main())



from __future__ import annotations

import asyncio
import os
import sys
import time
from datetime import datetime
from pathlib import Path
from types import SimpleNamespace

import pytest
from aiogram.exceptions import TelegramBadRequest
from aiogram.fsm.context import FSMContext
from aiogram.fsm.storage.base import StorageKey
from aiogram.fsm.storage.memory import MemoryStorage
from aiogram.types import ReplyKeyboardMarkup

ROOT = Path(__file__).resolve().parents[1]
if str(ROOT) not in sys.path:
    sys.path.insert(0, str(ROOT))

os.environ.setdefault("BOT_TOKEN", "TEST_TOKEN")

import bot  # noqa: E402
from parallel_runtime import BranchRef, CommonBranchRef, build_parallel_commit_message  # noqa: E402
from tasks import TaskRecord  # noqa: E402


class DummyMessage:
    def __init__(self, *, chat_id: int = 1, user_id: int = 1):
        self.calls = []
        self.edits = []
        self.chat = SimpleNamespace(id=chat_id)
        self.from_user = SimpleNamespace(id=user_id, full_name="Tester")
        self.message_id = 100
        self.date = datetime.now(bot.UTC)
        self.text = None
        self.caption = None

    async def answer(self, text: str, parse_mode=None, reply_markup=None, **kwargs):
        self.calls.append((text, parse_mode, reply_markup, kwargs))
        return SimpleNamespace(message_id=self.message_id + len(self.calls), chat=self.chat)

    async def edit_text(self, text: str, parse_mode=None, reply_markup=None, **kwargs):
        self.edits.append((text, parse_mode, reply_markup, kwargs))
        return SimpleNamespace(message_id=self.message_id, chat=self.chat)


class DummyCallback:
    def __init__(self, data: str, message: DummyMessage):
        self.data = data
        self.message = message
        self.answers = []
        self.from_user = SimpleNamespace(id=1, full_name="Tester")

    async def answer(self, text: str | None = None, show_alert: bool = False):
        self.answers.append((text, show_alert))


def make_state(message: DummyMessage) -> tuple[FSMContext, MemoryStorage]:
    storage = MemoryStorage()
    state = FSMContext(
        storage=storage,
        key=StorageKey(bot_id=999, chat_id=message.chat.id, user_id=message.from_user.id),
    )
    return state, storage


def _task() -> TaskRecord:
    return TaskRecord(
        id="TASK_0001",
        project_slug="demo",
        title="调研任务",
        status="research",
        priority=3,
        task_type="requirement",
        tags=(),
        due_date=None,
        description="需要调研的事项",
        parent_id=None,
        root_id="TASK_0001",
        depth=0,
        lineage="0001",
        created_at="2025-01-01T00:00:00+08:00",
        updated_at="2025-01-01T00:00:00+08:00",
        archived=False,
    )


def test_push_model_starts_with_dispatch_target_choice(monkeypatch):
    message = DummyMessage()
    callback = DummyCallback("task:push_model:TASK_0001", message)
    state, _storage = make_state(message)

    async def fake_get_task(task_id: str):
        assert task_id == "TASK_0001"
        return _task()

    monkeypatch.setattr(bot.TASK_SERVICE, "get_task", fake_get_task)

    async def _scenario() -> None:
        await bot.on_task_push_model(callback, state)
        assert await state.get_state() == bot.TaskPushStates.waiting_dispatch_target.state
        assert callback.answers and "请选择处理方式" in (callback.answers[-1][0] or "")
        assert message.calls, "应提示用户选择当前 CLI 或并行 CLI"
        prompt_text, _, reply_markup, _ = message.calls[-1]
        assert prompt_text == bot._build_push_dispatch_target_prompt()
        assert isinstance(reply_markup, ReplyKeyboardMarkup)

    asyncio.run(_scenario())


def test_model_quick_reply_keyboard_includes_parallel_actions():
    markup = bot._build_model_quick_reply_keyboard(
        task_id="TASK_0001",
        parallel_task_title="调研任务",
        enable_parallel_actions=True,
    )
    callback_data = [
        button.callback_data
        for row in markup.inline_keyboard
        for button in row
        if button.callback_data
    ]
    assert "task:detail:TASK_0001" in callback_data
    assert f"{bot.PARALLEL_REPLY_CALLBACK_PREFIX}TASK_0001" in callback_data
    assert f"{bot.PARALLEL_COMMIT_CALLBACK_PREFIX}TASK_0001" in callback_data


def test_parallel_reply_mode_auto_prefixes_next_message(monkeypatch):
    origin = DummyMessage()
    callback = DummyCallback(f"{bot.PARALLEL_REPLY_CALLBACK_PREFIX}TASK_0001", origin)

    recorded: list[tuple[str, object]] = []

    async def fake_handle_request_input_custom_text_message(_message):
        return False

    async def fake_handle_command_trigger_message(_message, _prompt, _state):
        return False

    async def fake_handle_prompt_dispatch(_message, prompt: str, **kwargs):
        recorded.append((prompt, kwargs.get("dispatch_context")))

    monkeypatch.setattr(bot, "_handle_request_input_custom_text_message", fake_handle_request_input_custom_text_message)
    monkeypatch.setattr(bot, "_handle_command_trigger_message", fake_handle_command_trigger_message)
    monkeypatch.setattr(bot, "_handle_prompt_dispatch", fake_handle_prompt_dispatch)

    dispatch_context = bot.ParallelDispatchContext(
        task_id="TASK_0001",
        tmux_session="vibe-par-demo",
        pointer_file=Path("/tmp/demo-pointer.txt"),
        workspace_root=Path("/tmp/demo-workspace"),
    )

    async def fake_active_parallel_session(task_id: str):
        return {
            "task_id": task_id,
            "tmux_session": dispatch_context.tmux_session,
            "pointer_file": str(dispatch_context.pointer_file),
            "workspace_root": str(dispatch_context.workspace_root),
        }

    monkeypatch.setattr(bot, "_get_active_parallel_session_for_task", fake_active_parallel_session)

    async def _scenario() -> None:
        await bot.on_parallel_reply_callback(callback)
        assert origin.calls and origin.calls[-1][0] == "已进入 /TASK_0001 回复模式。"
        reply_keyboard = origin.calls[-1][2]
        labels = [button.text for row in reply_keyboard.keyboard for button in row]
        assert labels == ["取消"]

        message = DummyMessage(chat_id=origin.chat.id, user_id=origin.from_user.id)
        message.text = "继续完善方案"
        state, _storage = make_state(message)
        await bot.on_text(message, state)

        assert recorded == [("继续完善方案", dispatch_context)]
        assert origin.chat.id not in bot.CHAT_PARALLEL_REPLY_TARGETS

    asyncio.run(_scenario())


def test_parallel_commit_message_uses_task_type_prefix():
    subject, body = build_parallel_commit_message(_task(), repo_name="web-base")
    assert subject == "feat(TASK_0001): 调研任务"
    assert "任务编码: /TASK_0001" in body
    assert "任务标题: 调研任务" in body
    assert "仓库: web-base" in body


def test_parallel_branch_title_shows_current_branch():
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=None,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [
                    BranchRef(name="develop", source="local", is_current=True),
                    BranchRef(name="feature/demo", source="local"),
                    BranchRef(name="origin/develop", source="remote", remote="origin"),
                ],
            )
        ],
        selections={},
        current_branch_labels={"backend-java": "develop"},
    )

    title = bot._build_parallel_branch_title(session, 0)

    assert "当前仓库：backend-java" in title
    assert "当前分支：develop" in title


def test_parallel_branch_keyboard_marks_current_branch_and_keeps_it_first():
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=None,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [
                    BranchRef(name="develop", source="local", is_current=True),
                    BranchRef(name="feature/demo", source="local"),
                    BranchRef(name="origin/develop", source="remote", remote="origin"),
                ],
            )
        ],
        selections={},
        current_branch_labels={"backend-java": "develop"},
    )

    markup = bot._build_parallel_branch_keyboard(session, repo_index=0, page=0)
    branch_rows = [row for row in markup.inline_keyboard if row and row[0].callback_data and row[0].callback_data.startswith(bot.PARALLEL_BRANCH_SELECT_PREFIX)]

    assert branch_rows[0][0].text.startswith("📍 develop")
    assert "（当前）" in branch_rows[0][0].text


def test_parallel_common_branch_keyboard_marks_current_and_has_individual_entry():
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=None,
        push_mode=None,
        supplement=None,
        repo_options=[],
        selections={},
        current_branch_labels={},
        common_branch_options=[
            CommonBranchRef(name="develop", source="local", current_count=2, total_repos=2),
            CommonBranchRef(name="origin/develop", source="remote", remote="origin", current_count=0, total_repos=2),
        ],
        selection_mode="bulk",
    )

    markup = bot._build_parallel_common_branch_keyboard(session, page=0)
    labels = [button.text for row in markup.inline_keyboard for button in row if button.text]

    assert any(label.startswith("📍 develop") and "（当前）" in label for label in labels)
    assert "🧩 逐个选择" in labels


def test_begin_parallel_launch_prefers_common_branch_selector(monkeypatch):
    message = DummyMessage()

    monkeypatch.setattr(
        bot,
        "discover_git_repos",
        lambda _base_dir, include_nested=False: [
            ("backend-java", Path("/tmp/backend-java"), "backend-java"),
            ("frontend-admin", Path("/tmp/frontend-admin"), "frontend-admin"),
        ],
    )
    monkeypatch.setattr(bot, "get_current_branch_state", lambda _repo_path: ("develop", "develop"))
    monkeypatch.setattr(
        bot,
        "list_branch_refs",
        lambda _repo_path, current_local_branch=None: [
            BranchRef(name="develop", source="local", is_current=True),
            BranchRef(name="origin/develop", source="remote", remote="origin"),
        ],
    )

    asyncio.run(
        bot._begin_parallel_launch(
            task=_task(),
            chat_id=message.chat.id,
            origin_message=message,
            actor=None,
            push_mode=None,
            supplement=None,
        )
    )

    assert message.calls, "首屏应展示共同分支批量选择"
    assert "共用的基线分支" in message.calls[-1][0]


def test_begin_parallel_launch_ignores_local_only_root_repo_in_common_branch_scope(monkeypatch):
    message = DummyMessage()

    root_path = Path("/tmp/workspace-root")
    backend_path = Path("/tmp/backend-java")
    frontend_path = Path("/tmp/web-base")

    monkeypatch.setattr(
        bot,
        "discover_git_repos",
        lambda _base_dir, include_nested=False: [
            ("__root__", root_path, "."),
            ("backend-java", backend_path, "backend-java"),
            ("web-base", frontend_path, "web-base"),
        ],
    )

    def fake_current_branch(repo_path: Path):
        if repo_path == root_path:
            return ("main", "main")
        return ("master", "master")

    def fake_list_branch_refs(repo_path: Path, current_local_branch=None):
        if repo_path == root_path:
            return [BranchRef(name="main", source="local", is_current=True)]
        return [
            BranchRef(name="master", source="local", is_current=True),
            BranchRef(name="origin/develop", source="remote", remote="origin"),
            BranchRef(name="origin/master", source="remote", remote="origin"),
        ]

    monkeypatch.setattr(bot, "get_current_branch_state", fake_current_branch)
    monkeypatch.setattr(bot, "list_branch_refs", fake_list_branch_refs)

    asyncio.run(
        bot._begin_parallel_launch(
            task=_task(),
            chat_id=message.chat.id,
            origin_message=message,
            actor=None,
            push_mode=None,
            supplement=None,
        )
    )

    assert message.calls, "忽略本地根仓库后，应进入共同分支批量选择"
    prompt_text = message.calls[-1][0]
    assert "共用的基线分支" in prompt_text
    assert "共同分支计算范围：2/3 个仓库" in prompt_text
    assert "已忽略仓库：.（根仓库无远端分支）" in prompt_text


def test_parallel_common_branch_select_populates_all_repo_selections(monkeypatch):
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [
                    BranchRef(name="develop", source="local", is_current=True),
                    BranchRef(name="origin/develop", source="remote", remote="origin"),
                ],
            ),
            (
                "frontend-admin",
                Path("/tmp/frontend-admin"),
                "frontend-admin",
                [
                    BranchRef(name="develop", source="local", is_current=True),
                    BranchRef(name="origin/develop", source="remote", remote="origin"),
                ],
            ),
        ],
        selections={},
        current_branch_labels={"backend-java": "develop", "frontend-admin": "develop"},
        common_branch_options=[CommonBranchRef(name="develop", source="local", current_count=2, total_repos=2)],
        selection_mode="bulk",
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    callback = DummyCallback(f"{bot.PARALLEL_COMMON_BRANCH_SELECT_PREFIX}demo:0", message)
    asyncio.run(bot.on_parallel_common_branch_select_callback(callback))

    assert session.selection_mode == "bulk"
    assert set(session.selections) == {"backend-java", "frontend-admin"}
    assert all(branch.name == "develop" for branch in session.selections.values())
    assert message.edits, "批量选择后应直接进入摘要确认页"
    assert "以下仓库将进入并行处理" in message.edits[-1][0]


def test_parallel_common_branch_select_redirects_to_first_unselected_repo_when_ignored_root_exists():
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "__root__",
                Path("/tmp/workspace-root"),
                ".",
                [BranchRef(name="main", source="local", is_current=True)],
            ),
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [
                    BranchRef(name="develop", source="local", is_current=True),
                    BranchRef(name="origin/develop", source="remote", remote="origin"),
                ],
            ),
            (
                "frontend-admin",
                Path("/tmp/frontend-admin"),
                "frontend-admin",
                [
                    BranchRef(name="develop", source="local", is_current=True),
                    BranchRef(name="origin/develop", source="remote", remote="origin"),
                ],
            ),
        ],
        selections={},
        current_branch_labels={"__root__": "main", "backend-java": "develop", "frontend-admin": "develop"},
        common_branch_options=[CommonBranchRef(name="develop", source="local", current_count=2, total_repos=2)],
        common_branch_repo_keys={"backend-java", "frontend-admin"},
        common_branch_scope_repo_count=2,
        common_branch_ignored_repos=["."],
        selection_mode="bulk",
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    callback = DummyCallback(f"{bot.PARALLEL_COMMON_BRANCH_SELECT_PREFIX}demo:0", message)
    asyncio.run(bot.on_parallel_common_branch_select_callback(callback))

    assert callback.answers[-1] == ("已批量应用共同分支 develop，请继续补选剩余仓库。", False)
    assert session.selection_mode == "individual"
    assert set(session.selections) == {"backend-java", "frontend-admin"}
    assert message.edits, "存在待补选仓库时，应直接进入逐个补选页"
    assert "当前仓库：." in message.edits[-1][0]


def test_parallel_branch_individual_callback_edits_first_repo():
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        common_branch_options=[CommonBranchRef(name="develop", source="local", current_count=1, total_repos=1)],
        selection_mode="bulk",
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    callback = DummyCallback(f"{bot.PARALLEL_BRANCH_INDIVIDUAL_PREFIX}demo", message)
    asyncio.run(bot.on_parallel_branch_individual_callback(callback))

    assert session.selection_mode == "individual"
    assert session.selections == {"backend-java": BranchRef(name="develop", source="local")}
    assert message.edits, "全部已选时，应直接回到摘要确认页"
    assert "以下仓库将进入并行处理" in message.edits[-1][0]


def test_parallel_branch_individual_callback_keeps_bulk_selections_and_starts_from_first_unselected_repo():
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "__root__",
                Path("/tmp/workspace-root"),
                ".",
                [BranchRef(name="main", source="local", is_current=True)],
            ),
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local", is_current=True)],
            ),
        ],
        selections={"backend-java": BranchRef(name="develop", source="local", is_current=True)},
        current_branch_labels={"__root__": "main", "backend-java": "develop"},
        common_branch_options=[CommonBranchRef(name="develop", source="local", current_count=1, total_repos=1)],
        common_branch_scope_repo_count=1,
        common_branch_ignored_repos=["."],
        selection_mode="bulk",
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    callback = DummyCallback(f"{bot.PARALLEL_BRANCH_INDIVIDUAL_PREFIX}demo", message)
    asyncio.run(bot.on_parallel_branch_individual_callback(callback))

    assert session.selection_mode == "individual"
    assert set(session.selections) == {"backend-java"}
    assert message.edits, "应从首个未选择仓库开始补选"
    assert "当前仓库：." in message.edits[-1][0]


def test_parallel_branch_page_callback_edits_same_message():
    message = DummyMessage()
    callback = DummyCallback("parallel:branch_page:demo:0:1", message)
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [
                    BranchRef(name=f"feature/{idx}", source="local")
                    for idx in range(12)
                ],
            )
        ],
        selections={},
        current_branch_labels={"backend-java": "feature/0"},
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    asyncio.run(bot.on_parallel_branch_page_callback(callback))

    assert message.edits, "翻页应编辑同一条消息"
    assert not message.calls, "翻页不应新增消息"


def test_parallel_branch_select_callback_edits_same_message_for_next_repo_and_summary():
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            ),
            (
                "frontend-admin",
                Path("/tmp/frontend-admin"),
                "frontend-admin",
                [BranchRef(name="main", source="local")],
            ),
        ],
        selections={},
        current_branch_labels={"backend-java": "develop", "frontend-admin": "main"},
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    callback_first = DummyCallback("parallel:branch_select:demo:0:0", message)
    asyncio.run(bot.on_parallel_branch_select_callback(callback_first))
    assert message.edits, "选择后进入下一仓库应编辑消息"
    assert not message.calls, "进入下一仓库不应新增消息"

    callback_second = DummyCallback("parallel:branch_select:demo:1:0", message)
    asyncio.run(bot.on_parallel_branch_select_callback(callback_second))
    assert len(message.edits) >= 2, "最后展示摘要也应继续编辑同一条消息"
    assert not message.calls, "展示开始处理按钮不应新增消息"


def test_parallel_branch_select_callback_skips_already_selected_repos_and_finishes_with_summary():
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "__root__",
                Path("/tmp/workspace-root"),
                ".",
                [BranchRef(name="main", source="local", is_current=True)],
            ),
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local", is_current=True)],
            ),
            (
                "frontend-admin",
                Path("/tmp/frontend-admin"),
                "frontend-admin",
                [BranchRef(name="develop", source="local", is_current=True)],
            ),
        ],
        selections={
            "backend-java": BranchRef(name="develop", source="local", is_current=True),
            "frontend-admin": BranchRef(name="develop", source="local", is_current=True),
        },
        current_branch_labels={"__root__": "main", "backend-java": "develop", "frontend-admin": "develop"},
        selection_mode="individual",
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    callback = DummyCallback("parallel:branch_select:demo:0:0", message)
    asyncio.run(bot.on_parallel_branch_select_callback(callback))

    assert session.selections["__root__"].name == "main"
    assert message.edits, "补选完成后应直接进入摘要页"
    assert "以下仓库将进入并行处理" in message.edits[-1][0]
    assert "当前仓库：" not in message.edits[-1][0]


def test_parallel_branch_confirm_callback_keeps_session_when_selection_incomplete():
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "__root__",
                Path("/tmp/workspace-root"),
                ".",
                [BranchRef(name="main", source="local", is_current=True)],
            ),
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local", is_current=True)],
            ),
        ],
        selections={"backend-java": BranchRef(name="develop", source="local", is_current=True)},
        current_branch_labels={"__root__": "main", "backend-java": "develop"},
        selection_mode="individual",
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    callback = DummyCallback("parallel:branch_confirm:demo", message)
    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))

    assert callback.answers[-1] == ("仍有仓库未选择基线分支", True)
    assert "demo" in bot.PARALLEL_LAUNCH_SESSIONS
    assert not message.edits


def test_parallel_branch_confirm_callback_edits_same_message_to_processing(monkeypatch, tmp_path: Path):
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    monkeypatch.setattr(
        bot,
        "prepare_parallel_workspace",
        lambda **_kwargs: [],
    )

    async def fake_start_parallel_tmux_session(task, workspace_root):
        return "vibe-par-demo", tmp_path / "pointer.txt"

    async def fake_upsert_session(**_kwargs):
        return None

    async def fake_push_task_to_model(*_args, **_kwargs):
        return True, "PROMPT", tmp_path / "session.jsonl"

    async def fake_send_preview(*_args, **_kwargs):
        return None

    async def fake_send_ack(*_args, **_kwargs):
        return None

    monkeypatch.setattr(bot, "_start_parallel_tmux_session", fake_start_parallel_tmux_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "upsert_session", fake_upsert_session)
    monkeypatch.setattr(bot, "_push_task_to_model", fake_push_task_to_model)
    monkeypatch.setattr(bot, "_send_model_push_preview", fake_send_preview)
    monkeypatch.setattr(bot, "_send_session_ack", fake_send_ack)

    callback = DummyCallback("parallel:branch_confirm:demo", message)
    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))

    assert message.edits, "点击开始并行处理应先覆盖原消息为处理中提示"
    assert callback.answers == [(None, False)]
    assert message.edits[0][0] == "正在准备并行副本，请稍候……"


def test_parallel_branch_confirm_callback_replaces_processing_message_with_summary(monkeypatch, tmp_path: Path):
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    monkeypatch.setattr(bot, "prepare_parallel_workspace", lambda **_kwargs: [])

    async def fake_start_parallel_tmux_session(task, workspace_root):
        return "vibe-par-demo", tmp_path / "pointer.txt"

    async def fake_upsert_session(**_kwargs):
        return None

    async def fake_push_task_to_model(*_args, **_kwargs):
        return True, "PROMPT", tmp_path / "session.jsonl"

    async def fake_send_preview(*_args, **_kwargs):
        return None

    async def fake_send_ack(*_args, **_kwargs):
        return None

    monkeypatch.setattr(bot, "_start_parallel_tmux_session", fake_start_parallel_tmux_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "upsert_session", fake_upsert_session)
    monkeypatch.setattr(bot, "_push_task_to_model", fake_push_task_to_model)
    monkeypatch.setattr(bot, "_send_model_push_preview", fake_send_preview)
    monkeypatch.setattr(bot, "_send_session_ack", fake_send_ack)

    callback = DummyCallback("parallel:branch_confirm:demo", message)
    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))

    assert len(message.edits) >= 3, "成功后应继续覆盖同一条消息为阶段提示与摘要"
    assert message.edits[0][0] == "正在准备并行副本，请稍候……"
    assert any(text == "正在启动并行 CLI，请稍候……" for text, *_rest in message.edits)
    assert "已创建并行开发副本（原目录未改动）：" in message.edits[-1][0]
    assert not message.calls, "成功收口不应再额外新增摘要消息"


def test_parallel_branch_confirm_callback_passes_source_root_for_full_copy(monkeypatch, tmp_path: Path):
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/project-root/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        base_dir=Path("/tmp/project-root"),
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    captured: dict[str, object] = {}

    def fake_prepare_parallel_workspace(**kwargs):
        captured.update(kwargs)
        return []

    monkeypatch.setattr(bot, "prepare_parallel_workspace", fake_prepare_parallel_workspace)

    async def fake_start_parallel_tmux_session(task, workspace_root):
        return "vibe-par-demo", tmp_path / "pointer.txt"

    async def fake_upsert_session(**_kwargs):
        return None

    async def fake_push_task_to_model(*_args, **_kwargs):
        return True, "PROMPT", tmp_path / "session.jsonl"

    async def fake_send_preview(*_args, **_kwargs):
        return None

    async def fake_send_ack(*_args, **_kwargs):
        return None

    monkeypatch.setattr(bot, "_start_parallel_tmux_session", fake_start_parallel_tmux_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "upsert_session", fake_upsert_session)
    monkeypatch.setattr(bot, "_push_task_to_model", fake_push_task_to_model)
    monkeypatch.setattr(bot, "_send_model_push_preview", fake_send_preview)
    monkeypatch.setattr(bot, "_send_session_ack", fake_send_ack)

    callback = DummyCallback("parallel:branch_confirm:demo", message)
    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))

    assert captured["source_root"] == Path("/tmp/project-root")


def test_parallel_branch_confirm_callback_prompts_for_prefix_when_missing():
    """仓库分支全部选完后，首次确认应先进入分支前缀输入，而不是直接创建并行目录。"""

    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=message.chat.id,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    callback = DummyCallback("parallel:branch_confirm:demo", message)
    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))

    assert callback.answers[-1] == ("请输入分支前缀；发送取消将使用默认前缀", False)
    assert bot.CHAT_PARALLEL_BRANCH_PREFIX_INPUTS[message.chat.id] == "demo"
    assert message.calls, "应发送新消息进入前缀输入页并展示底部菜单按钮"
    prompt_text, _, reply_markup, _ = message.calls[-1]
    assert "请输入本次并行任务的分支前缀（例如：Sprint001）。" in prompt_text
    assert "示例（自定义）：Sprint001/TASK\\_0001-调研任务" in prompt_text
    assert "示例（默认）：vibego/TASK\\_0001-调研任务" in prompt_text
    labels = [button.text for row in reply_markup.keyboard for button in row]
    assert labels == ["取消"]


def test_parallel_branch_prefix_cancel_uses_default_prefix(monkeypatch, tmp_path: Path):
    """前缀输入页点击取消时，应回退默认前缀并继续创建并行目录。"""

    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=message.chat.id,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session
    bot.CHAT_PARALLEL_BRANCH_PREFIX_INPUTS[message.chat.id] = "demo"

    captured: dict[str, object] = {}

    def fake_prepare_parallel_workspace(**kwargs):
        captured.update(kwargs)
        return []

    monkeypatch.setattr(bot, "prepare_parallel_workspace", fake_prepare_parallel_workspace)

    async def fake_start_parallel_tmux_session(task, workspace_root):
        return "vibe-par-demo", tmp_path / "pointer.txt"

    async def fake_upsert_session(**_kwargs):
        return None

    async def fake_push_task_to_model(*_args, **_kwargs):
        return True, "PROMPT", tmp_path / "session.jsonl"

    async def fake_send_preview(*_args, **_kwargs):
        return None

    async def fake_send_ack(*_args, **_kwargs):
        return None

    monkeypatch.setattr(bot, "_start_parallel_tmux_session", fake_start_parallel_tmux_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "upsert_session", fake_upsert_session)
    monkeypatch.setattr(bot, "_push_task_to_model", fake_push_task_to_model)
    monkeypatch.setattr(bot, "_send_model_push_preview", fake_send_preview)
    monkeypatch.setattr(bot, "_send_session_ack", fake_send_ack)

    callback = DummyCallback(f"{bot.PARALLEL_BRANCH_PREFIX_CANCEL_PREFIX}demo", message)
    asyncio.run(bot.on_parallel_branch_prefix_cancel_callback(callback))

    assert captured["branch_prefix"] == bot.DEFAULT_PARALLEL_BRANCH_PREFIX
    assert session.branch_prefix == bot.DEFAULT_PARALLEL_BRANCH_PREFIX
    assert message.chat.id not in bot.CHAT_PARALLEL_BRANCH_PREFIX_INPUTS


def test_parallel_branch_prefix_text_uses_custom_prefix_and_continues(monkeypatch, tmp_path: Path):
    """输入自定义前缀后，应以 prefix/TASK_... 创建任务分支。"""

    origin = DummyMessage(chat_id=9, user_id=9)
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=origin.chat.id,
        actor=None,
        origin_message=origin,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session
    bot.CHAT_PARALLEL_BRANCH_PREFIX_INPUTS[origin.chat.id] = "demo"

    captured: dict[str, object] = {}

    def fake_prepare_parallel_workspace(**kwargs):
        captured.update(kwargs)
        return []

    monkeypatch.setattr(bot, "prepare_parallel_workspace", fake_prepare_parallel_workspace)

    async def fake_start_parallel_tmux_session(task, workspace_root):
        return "vibe-par-demo", tmp_path / "pointer.txt"

    async def fake_upsert_session(**_kwargs):
        return None

    async def fake_push_task_to_model(*_args, **_kwargs):
        return True, "PROMPT", tmp_path / "session.jsonl"

    async def fake_send_preview(*_args, **_kwargs):
        return None

    async def fake_send_ack(*_args, **_kwargs):
        return None

    monkeypatch.setattr(bot, "_start_parallel_tmux_session", fake_start_parallel_tmux_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "upsert_session", fake_upsert_session)
    monkeypatch.setattr(bot, "_push_task_to_model", fake_push_task_to_model)
    monkeypatch.setattr(bot, "_send_model_push_preview", fake_send_preview)
    monkeypatch.setattr(bot, "_send_session_ack", fake_send_ack)

    message = DummyMessage(chat_id=origin.chat.id, user_id=origin.from_user.id)
    message.text = "TRADE114"
    state, _storage = make_state(message)

    asyncio.run(bot.on_text(message, state))

    assert captured["branch_prefix"] == "TRADE114"
    assert session.branch_prefix == "TRADE114"
    assert origin.chat.id not in bot.CHAT_PARALLEL_BRANCH_PREFIX_INPUTS


def test_parallel_branch_confirm_callback_acknowledges_callback_before_prepare(monkeypatch, tmp_path: Path):
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    callback = DummyCallback("parallel:branch_confirm:demo", message)

    def fake_prepare_parallel_workspace(**_kwargs):
        assert callback.answers == [(None, False)], "进入重操作前应先确认 callback，避免超时失效"
        return []

    async def fake_start_parallel_tmux_session(task, workspace_root):
        return "vibe-par-demo", tmp_path / "pointer.txt"

    async def fake_upsert_session(**_kwargs):
        return None

    async def fake_push_task_to_model(*_args, **_kwargs):
        return True, "PROMPT", tmp_path / "session.jsonl"

    async def fake_send_preview(*_args, **_kwargs):
        return None

    async def fake_send_ack(*_args, **_kwargs):
        return None

    monkeypatch.setattr(bot, "prepare_parallel_workspace", fake_prepare_parallel_workspace)
    monkeypatch.setattr(bot, "_start_parallel_tmux_session", fake_start_parallel_tmux_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "upsert_session", fake_upsert_session)
    monkeypatch.setattr(bot, "_push_task_to_model", fake_push_task_to_model)
    monkeypatch.setattr(bot, "_send_model_push_preview", fake_send_preview)
    monkeypatch.setattr(bot, "_send_session_ack", fake_send_ack)

    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))


def test_parallel_branch_confirm_callback_falls_back_to_chat_when_failure_callback_expires(monkeypatch):
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    async def flaky_answer(text: str | None = None, show_alert: bool = False):
        if text == "并行创建失败":
            raise TelegramBadRequest(method="answerCallbackQuery", message="Bad Request: query is too old")
        callback.answers.append((text, show_alert))

    monkeypatch.setattr(
        bot,
        "prepare_parallel_workspace",
        lambda **_kwargs: (_ for _ in ()).throw(RuntimeError("ssh: connect to host gitlab.cckggroup.com port 10002: Connection reset by peer")),
    )

    callback = DummyCallback("parallel:branch_confirm:demo", message)
    monkeypatch.setattr(callback, "answer", flaky_answer)

    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))

    assert callback.answers == [(None, False)]
    assert message.calls, "callback 失效后仍应回退为普通聊天消息"
    assert "并行创建失败：" in message.calls[-1][0]
    assert "Connection reset by peer" in message.calls[-1][0]


def test_parallel_branch_confirm_callback_reports_prepare_timeout(monkeypatch):
    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=None,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    def slow_prepare_parallel_workspace(**_kwargs):
        time.sleep(0.05)
        return []

    async def should_not_start_tmux(*_args, **_kwargs):
        raise AssertionError("准备阶段超时后不应继续启动并行 CLI")

    monkeypatch.setattr(bot, "prepare_parallel_workspace", slow_prepare_parallel_workspace)
    monkeypatch.setattr(bot, "PARALLEL_WORKSPACE_PREPARE_TIMEOUT_SECONDS", 0.01, raising=False)
    monkeypatch.setattr(bot, "_start_parallel_tmux_session", should_not_start_tmux)

    callback = DummyCallback("parallel:branch_confirm:demo", message)
    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))

    assert callback.answers == [(None, False)]
    assert message.calls, "准备超时后应向聊天明确回报失败"
    assert "并行副本准备超时" in message.calls[-1][0]


def test_parallel_branch_confirm_callback_stops_when_push_task_to_model_fails(monkeypatch, tmp_path: Path):
    """并行 tmux 已启动但首次推送失败时，不得继续展示成功摘要或预览。"""

    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=bot.PUSH_MODE_PLAN,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    monkeypatch.setattr(bot, "prepare_parallel_workspace", lambda **_kwargs: [])

    async def fake_start_parallel_tmux_session(task, workspace_root):
        return "vibe-par-demo", tmp_path / "pointer.txt"

    async def fake_upsert_session(**_kwargs):
        return None

    updates: list[tuple[str, dict]] = []

    async def fake_update_status(task_id: str, **kwargs):
        updates.append((task_id, kwargs))

    async def fake_push_task_to_model(*_args, **_kwargs):
        return False, "PROMPT", None

    preview_calls: list[int] = []
    ack_calls: list[int] = []

    async def fake_send_preview(*_args, **_kwargs):
        preview_calls.append(1)
        return None

    async def fake_send_ack(*_args, **_kwargs):
        ack_calls.append(1)
        return None

    monkeypatch.setattr(bot, "_start_parallel_tmux_session", fake_start_parallel_tmux_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "upsert_session", fake_upsert_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "update_status", fake_update_status)
    monkeypatch.setattr(bot, "_push_task_to_model", fake_push_task_to_model)
    monkeypatch.setattr(bot, "_send_model_push_preview", fake_send_preview)
    monkeypatch.setattr(bot, "_send_session_ack", fake_send_ack)

    callback = DummyCallback("parallel:branch_confirm:demo", message)
    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))

    assert callback.answers == [(None, False)]
    assert message.calls, "首次推送失败后应向聊天明确回报失败"
    assert "并行 CLI 未启动成功" in message.calls[-1][0]
    assert preview_calls == []
    assert ack_calls == []
    assert not any("已创建并行开发副本（原目录未改动）：" in text for text, *_rest in message.edits)
    assert updates and updates[-1][1]["status"] == "closed"


def test_parallel_branch_confirm_callback_ensures_workspace_trust_before_starting_tmux(monkeypatch, tmp_path: Path):
    """启动并行 tmux 前应先确保 workspace 已写入 Codex trusted 配置。"""

    message = DummyMessage()
    session = bot.ParallelLaunchSession(
        token="demo",
        task=_task(),
        chat_id=1,
        actor=None,
        origin_message=message,
        push_mode=bot.PUSH_MODE_PLAN,
        supplement=None,
        repo_options=[
            (
                "backend-java",
                Path("/tmp/backend-java"),
                "backend-java",
                [BranchRef(name="develop", source="local")],
            )
        ],
        selections={"backend-java": BranchRef(name="develop", source="local")},
        current_branch_labels={"backend-java": "develop"},
        branch_prefix=bot.DEFAULT_PARALLEL_BRANCH_PREFIX,
    )
    bot.PARALLEL_LAUNCH_SESSIONS["demo"] = session

    monkeypatch.setattr(bot, "prepare_parallel_workspace", lambda **_kwargs: [])

    call_order: list[tuple[str, str]] = []

    async def fake_ensure_trusted(path: Path, *, scope: str, owner_key: str):
        call_order.append(("trust", f"{path}|{scope}|{owner_key}"))

    async def fake_start_parallel_tmux_session(task, workspace_root):
        call_order.append(("tmux", str(workspace_root)))
        return "vibe-par-demo", tmp_path / "pointer.txt"

    async def fake_upsert_session(**_kwargs):
        return None

    async def fake_push_task_to_model(*_args, **_kwargs):
        return True, "PROMPT", tmp_path / "session.jsonl"

    async def fake_send_preview(*_args, **_kwargs):
        return None

    async def fake_send_ack(*_args, **_kwargs):
        return None

    monkeypatch.setattr(bot, "_ensure_codex_trusted_project_path", fake_ensure_trusted)
    monkeypatch.setattr(bot, "_start_parallel_tmux_session", fake_start_parallel_tmux_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "upsert_session", fake_upsert_session)
    monkeypatch.setattr(bot, "_push_task_to_model", fake_push_task_to_model)
    monkeypatch.setattr(bot, "_send_model_push_preview", fake_send_preview)
    monkeypatch.setattr(bot, "_send_session_ack", fake_send_ack)

    callback = DummyCallback("parallel:branch_confirm:demo", message)
    asyncio.run(bot.on_parallel_branch_confirm_callback(callback))

    assert call_order[0][0] == "trust"
    assert call_order[1][0] == "tmux"


def test_start_parallel_tmux_session_requires_ready_file(monkeypatch, tmp_path: Path):
    """并行 CLI 启动脚本即使返回成功，也必须产出 ready 回执文件。"""

    task = SimpleNamespace(id="TASK_9005")
    workspace_root = tmp_path / "workspace"
    workspace_root.mkdir(parents=True, exist_ok=True)

    monkeypatch.setattr(bot, "CONFIG_ROOT_PATH", tmp_path)
    monkeypatch.setattr(bot, "PROJECT_SLUG", "demo")

    captured_env: dict[str, str] = {}

    class DummyProcess:
        returncode = 0

        async def communicate(self):
            return b"ok", b""

    async def fake_create_subprocess_exec(*cmd, **kwargs):
        captured_env.update({key: str(value) for key, value in kwargs["env"].items()})
        return DummyProcess()

    monkeypatch.setattr(bot.asyncio, "create_subprocess_exec", fake_create_subprocess_exec)

    async def scenario() -> None:
        with pytest.raises(RuntimeError, match="ready 回执"):
            await bot._start_parallel_tmux_session(task, workspace_root)

    asyncio.run(scenario())

    assert captured_env["SESSION_READY_FILE"].endswith("tmux_ready")


def test_start_parallel_tmux_session_returns_after_ready_file_written(monkeypatch, tmp_path: Path):
    """并行 CLI 启动脚本返回成功且 ready 回执存在时，应允许继续后续派发。"""

    task = SimpleNamespace(id="TASK_9006")
    workspace_root = tmp_path / "workspace"
    workspace_root.mkdir(parents=True, exist_ok=True)

    monkeypatch.setattr(bot, "CONFIG_ROOT_PATH", tmp_path)
    monkeypatch.setattr(bot, "PROJECT_SLUG", "demo")

    class DummyProcess:
        returncode = 0

        async def communicate(self):
            return b"ok", b""

    async def fake_create_subprocess_exec(*cmd, **kwargs):
        ready_file = Path(str(kwargs["env"]["SESSION_READY_FILE"]))
        ready_file.parent.mkdir(parents=True, exist_ok=True)
        ready_file.write_text("ready", encoding="utf-8")
        return DummyProcess()

    monkeypatch.setattr(bot.asyncio, "create_subprocess_exec", fake_create_subprocess_exec)

    async def scenario() -> None:
        tmux_session, pointer_file = await bot._start_parallel_tmux_session(task, workspace_root)
        assert tmux_session == "vibe-par-demo-task_9006"
        assert pointer_file.name == "current_session.txt"

    asyncio.run(scenario())


def test_parallel_commit_callback_reports_runtime_failure(monkeypatch):
    """提交并行分支异常时，应在 Telegram 明确回执失败原因。"""

    message = DummyMessage()
    callback = DummyCallback(f"{bot.PARALLEL_COMMIT_CALLBACK_PREFIX}TASK_0001", message)

    async def fake_active_parallel_session(_task_id: str):
        return SimpleNamespace(task_id="TASK_0001")

    async def fake_get_task(task_id: str):
        assert task_id == "TASK_0001"
        return _task()

    async def fake_get_repos(_task_id: str):
        return [SimpleNamespace(repo_key="repo")]

    async def fake_update_status(*_args, **_kwargs):
        return None

    async def fake_to_thread(*_args, **_kwargs):
        raise RuntimeError("git 输出解码失败")

    monkeypatch.setattr(bot, "_get_active_parallel_session_for_task", fake_active_parallel_session)
    monkeypatch.setattr(bot.TASK_SERVICE, "get_task", fake_get_task)
    monkeypatch.setattr(bot, "_get_parallel_session_repos", fake_get_repos)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "update_status", fake_update_status)
    monkeypatch.setattr(bot.asyncio, "to_thread", fake_to_thread)

    asyncio.run(bot.on_parallel_commit_callback(callback))

    assert callback.answers[0] == ("正在提交并行分支…", False)
    assert message.calls, "异常时应向聊天发送失败消息"
    text, _, reply_markup, _ = message.calls[-1]
    assert "提交失败" in text
    assert "git 输出解码失败" in text
    assert isinstance(reply_markup, ReplyKeyboardMarkup)


def test_parallel_merge_callback_reports_runtime_failure(monkeypatch):
    """自动合并异常时，应在 Telegram 明确回执失败原因。"""

    message = DummyMessage()
    callback = DummyCallback(f"{bot.PARALLEL_MERGE_CALLBACK_PREFIX}TASK_0001", message)

    async def fake_active_parallel_session(_task_id: str):
        return SimpleNamespace(task_id="TASK_0001")

    async def fake_get_task(task_id: str):
        assert task_id == "TASK_0001"
        return _task()

    async def fake_get_repos(_task_id: str):
        return [SimpleNamespace(repo_key="repo")]

    async def fake_update_status(*_args, **_kwargs):
        return None

    async def fake_to_thread(*_args, **_kwargs):
        raise RuntimeError("远端服务异常")

    monkeypatch.setattr(bot, "_get_active_parallel_session_for_task", fake_active_parallel_session)
    monkeypatch.setattr(bot.TASK_SERVICE, "get_task", fake_get_task)
    monkeypatch.setattr(bot, "_get_parallel_session_repos", fake_get_repos)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "update_status", fake_update_status)
    monkeypatch.setattr(bot.asyncio, "to_thread", fake_to_thread)

    asyncio.run(bot.on_parallel_merge_callback(callback))

    assert callback.answers[0] == ("正在尝试自动合并…", False)
    assert message.calls, "异常时应向聊天发送失败消息"
    text, _, reply_markup, _ = message.calls[-1]
    assert "自动合并失败" in text
    assert "远端服务异常" in text
    assert isinstance(reply_markup, ReplyKeyboardMarkup)



from __future__ import annotations

import subprocess
from pathlib import Path

import pytest
import parallel_runtime
from tasks import TaskRecord

from parallel_runtime import (
    BranchRef,
    RepoBranchSelection,
    build_parallel_branch_name,
    commit_parallel_repos,
    collect_common_branch_refs,
    delete_parallel_workspace,
    discover_git_repos,
    filter_common_branch_repo_options,
    get_current_branch_state,
    list_branch_refs,
    merge_parallel_repos,
    prepare_parallel_workspace,
)


def _git(repo: Path, *args: str) -> subprocess.CompletedProcess[str]:
    """执行 git 命令，失败时直接让测试暴露上下文。"""

    return subprocess.run(
        ["git", *args],
        cwd=repo,
        text=True,
        capture_output=True,
        check=True,
    )


def test_run_git_forces_utf8_replace(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> None:
    """git 子进程应固定使用 utf-8 + replace，避免系统 locale 导致解码崩溃。"""

    captured: dict[str, object] = {}

    def fake_run(args, **kwargs):
        captured["args"] = args
        captured["kwargs"] = kwargs
        return subprocess.CompletedProcess(args, 0, stdout="", stderr="")

    monkeypatch.setattr(parallel_runtime.subprocess, "run", fake_run)

    result = parallel_runtime._run_git(["status", "--short"], cwd=tmp_path)

    assert result.returncode == 0
    assert captured["args"] == ["git", "status", "--short"]
    assert captured["kwargs"]["cwd"] == str(tmp_path)
    assert captured["kwargs"]["text"] is True
    assert captured["kwargs"]["encoding"] == "utf-8"
    assert captured["kwargs"]["errors"] == "replace"
    env = captured["kwargs"]["env"]
    assert "http_proxy" not in env
    assert "https_proxy" not in env
    assert "HTTP_PROXY" not in env
    assert "HTTPS_PROXY" not in env
    assert "all_proxy" not in env
    assert "ALL_PROXY" not in env


def _init_repo(repo: Path, files: dict[str, str] | None = None) -> None:
    """初始化最小 Git 仓库，供并行运行时测试复用。"""

    repo.mkdir(parents=True, exist_ok=True)
    _git(repo, "init")
    _git(repo, "config", "user.name", "Test User")
    _git(repo, "config", "user.email", "test@example.com")
    for relative_path, content in (files or {"README.md": "# demo\n"}).items():
        file_path = repo / relative_path
        file_path.parent.mkdir(parents=True, exist_ok=True)
        file_path.write_text(content, encoding="utf-8")
    _git(repo, "add", "-A")
    _git(repo, "commit", "-m", "init")


def _current_branch(repo: Path) -> str:
    """读取仓库当前分支名，避免测试依赖 main/master 默认值。"""

    return _git(repo, "symbolic-ref", "--short", "HEAD").stdout.strip()


def _make_task(task_id: str = "TASK_0100", title: str = "并行工作区修复") -> TaskRecord:
    """构造并行提交测试所需的最小任务对象。"""

    return TaskRecord(
        id=task_id,
        project_slug="demo",
        title=title,
        status="research",
        priority=3,
        task_type="task",
        tags=(),
        due_date=None,
        description="测试并行运行时",
        parent_id=None,
        root_id=task_id,
        depth=0,
        lineage="0100",
        created_at="2025-01-01T00:00:00+08:00",
        updated_at="2025-01-01T00:00:00+08:00",
        archived=False,
    )


def _init_bare_remote(remote: Path) -> None:
    """初始化 bare 远端，供 push/merge 场景验证。"""

    remote.parent.mkdir(parents=True, exist_ok=True)
    subprocess.run(["git", "init", "--bare", str(remote)], check=True, capture_output=True, text=True)


def test_build_parallel_branch_name_uses_default_prefix() -> None:
    """未指定前缀时，应统一落到默认分支组。"""

    assert build_parallel_branch_name("TASK_0108", "类目编码校验") == "vibego/TASK_0108-类目编码校验"


def test_build_parallel_branch_name_uses_custom_prefix() -> None:
    """指定前缀时，应生成 prefix/TASK_... 结构。"""

    assert (
        build_parallel_branch_name("TASK_0109", "类目编码校验", prefix="TRADE114")
        == "TRADE114/TASK_0109-类目编码校验"
    )


def test_discover_git_repos_skips_nested_repos_under_non_root_parent(tmp_path: Path) -> None:
    """并行仓库发现应跳过已被父仓库覆盖的嵌套仓库，避免后续 clone 路径冲突。"""

    root = tmp_path / "mall"
    _init_repo(root, {"README.md": "root\n"})
    _init_repo(root / "backend-java", {"service.txt": "backend\n"})
    _init_repo(root / "frontend-admin", {"index.html": "<html></html>\n"})
    _init_repo(root / "backend-java" / "infra-parent", {"pom.xml": "<project />\n"})
    _init_repo(root / "backend-java" / "infra-common", {"pom.xml": "<project />\n"})

    repos = discover_git_repos(root)
    keys = [repo_key for repo_key, _repo_path, _relative_path in repos]

    assert "__root__" in keys
    assert "backend-java" in keys
    assert "frontend-admin" in keys
    assert "backend-java/infra-parent" not in keys
    assert "backend-java/infra-common" not in keys


def test_discover_git_repos_includes_nested_repos_when_requested(tmp_path: Path) -> None:
    """批量分支选择需要看到所有 Git 库时，应支持返回嵌套仓库。"""

    root = tmp_path / "mall"
    _init_repo(root, {"README.md": "root\n"})
    _init_repo(root / "backend-java", {"service.txt": "backend\n"})
    _init_repo(root / "backend-java" / "infra-parent", {"pom.xml": "<project />\n"})

    repos = discover_git_repos(root, include_nested=True)
    keys = [repo_key for repo_key, _repo_path, _relative_path in repos]

    assert "__root__" in keys
    assert "backend-java" in keys
    assert "backend-java/infra-parent" in keys


def test_prepare_parallel_workspace_rejects_overlapping_relative_paths(tmp_path: Path) -> None:
    """即使上游传入了父子重叠路径，也应给出明确错误而不是透传生硬的 git 报错。"""

    parent_repo = tmp_path / "sources" / "backend-java"
    nested_repo = tmp_path / "sources" / "infra-parent"
    _init_repo(parent_repo, {"infra-parent/pom.xml": "<project />\n"})
    _init_repo(nested_repo, {"pom.xml": "<project />\n"})

    with pytest.raises(RuntimeError, match="并行仓库路径重叠"):
        prepare_parallel_workspace(
            workspace_root=tmp_path / "workspace",
            task_id="TASK_0093",
            title="并行创建缺陷修复",
            selections=[
                RepoBranchSelection(
                    repo_key="backend-java",
                    source_repo_path=parent_repo,
                    selected_ref="HEAD",
                    selected_remote=None,
                    relative_path="backend-java",
                ),
                RepoBranchSelection(
                    repo_key="backend-java/infra-parent",
                    source_repo_path=nested_repo,
                    selected_ref="HEAD",
                    selected_remote=None,
                    relative_path="backend-java/infra-parent",
                ),
            ],
        )


def test_collect_common_branch_refs_builds_intersection_and_current_marker() -> None:
    """共同分支列表应只保留所有仓库共有的分支，并统计当前分支命中数。"""

    options = collect_common_branch_refs(
        [
            (
                "backend-java",
                [
                    BranchRef(name="develop", source="local", is_current=True),
                    BranchRef(name="release", source="local"),
                    BranchRef(name="origin/develop", source="remote", remote="origin"),
                ],
            ),
            (
                "frontend-admin",
                [
                    BranchRef(name="develop", source="local", is_current=True),
                    BranchRef(name="feature/demo", source="local"),
                    BranchRef(name="origin/develop", source="remote", remote="origin"),
                ],
            ),
        ]
    )

    assert [(item.name, item.source, item.current_count, item.total_repos) for item in options] == [
        ("develop", "local", 2, 2),
        ("origin/develop", "remote", 0, 2),
    ]


def test_filter_common_branch_repo_options_ignores_local_only_root_repo() -> None:
    """共同分支计算应忽略没有远端分支的本地根仓库。"""

    eligible, ignored = filter_common_branch_repo_options(
        [
            (
                "__root__",
                ".",
                [
                    BranchRef(name="main", source="local", is_current=True),
                ],
            ),
            (
                "backend-java",
                "backend-java",
                [
                    BranchRef(name="master", source="local", is_current=True),
                    BranchRef(name="origin/master", source="remote", remote="origin"),
                ],
            ),
            (
                "frontend-admin",
                "frontend-admin",
                [
                    BranchRef(name="master", source="local", is_current=True),
                    BranchRef(name="origin/master", source="remote", remote="origin"),
                ],
            ),
        ]
    )

    assert [repo_key for repo_key, _branches in eligible] == ["backend-java", "frontend-admin"]
    assert ignored == ["."]


def test_filter_common_branch_repo_options_keeps_root_repo_when_remote_exists() -> None:
    """根仓库一旦存在远端分支，就仍应参与共同分支计算。"""

    eligible, ignored = filter_common_branch_repo_options(
        [
            (
                "__root__",
                ".",
                [
                    BranchRef(name="main", source="local", is_current=True),
                    BranchRef(name="origin/master", source="remote", remote="origin"),
                ],
            ),
            (
                "backend-java",
                "backend-java",
                [
                    BranchRef(name="master", source="local", is_current=True),
                    BranchRef(name="origin/master", source="remote", remote="origin"),
                ],
            ),
        ]
    )

    assert [repo_key for repo_key, _branches in eligible] == ["__root__", "backend-java"]
    assert ignored == []


def test_prepare_parallel_workspace_copies_full_workdir_and_excludes_generated_dirs(tmp_path: Path) -> None:
    """整目录复刻时应保留普通/嵌套仓库内容，并排除生成物目录。"""

    source_root = tmp_path / "source"
    _init_repo(
        source_root,
        {
            "README.md": "root\n",
            ".gitignore": "target/\nlogs/\n.idea/\n",
        },
    )
    nested_repo = source_root / "service"
    _init_repo(
        nested_repo,
        {
            "src/main.py": "print('demo')\n",
            ".gitignore": "target/\nlogs/\n.idea/\n",
        },
    )
    (source_root / "notes.txt").write_text("需要完整复制\n", encoding="utf-8")
    (source_root / "target").mkdir()
    (source_root / "target" / "app.bin").write_text("bin\n", encoding="utf-8")
    (source_root / "logs").mkdir()
    (source_root / "logs" / "app.log").write_text("log\n", encoding="utf-8")
    (source_root / ".idea").mkdir()
    (source_root / ".idea" / "workspace.xml").write_text("<idea/>\n", encoding="utf-8")
    (source_root / "node_modules").mkdir()
    (source_root / "node_modules" / "demo.js").write_text("console.log('x')\n", encoding="utf-8")

    records = prepare_parallel_workspace(
        workspace_root=tmp_path / "workspace",
        task_id="TASK_0093",
        title="并行创建缺陷修复",
        source_root=source_root,
        selections=[
            RepoBranchSelection(
                repo_key="__root__",
                source_repo_path=source_root,
                selected_ref="HEAD",
                selected_remote=None,
                relative_path=".",
            ),
            RepoBranchSelection(
                repo_key="service",
                source_repo_path=nested_repo,
                selected_ref="HEAD",
                selected_remote=None,
                relative_path="service",
            ),
        ],
    )

    workspace_root = tmp_path / "workspace"
    assert (workspace_root / "notes.txt").read_text(encoding="utf-8") == "需要完整复制\n"
    assert (workspace_root / "service" / "src" / "main.py").exists()
    assert not (workspace_root / "target").exists()
    assert not (workspace_root / "logs").exists()
    assert not (workspace_root / ".idea" / "workspace.xml").exists()
    assert (workspace_root / ".idea" / "vcs.xml").exists()
    assert not (workspace_root / "node_modules").exists()
    assert len(records) == 2


def test_prepare_parallel_workspace_links_root_docs_to_source_docs(tmp_path: Path) -> None:
    """并行工作区根级 docs 应直接共享真实项目 docs，而不是保留副本。"""

    source_root = tmp_path / "source"
    _init_repo(source_root, {"README.md": "root\n"})
    (source_root / "docs").mkdir(parents=True, exist_ok=True)
    (source_root / "docs" / "TASK_0079.md").write_text("真实项目文档\n", encoding="utf-8")

    prepare_parallel_workspace(
        workspace_root=tmp_path / "workspace",
        task_id="TASK_0079",
        title="共享真实 docs",
        source_root=source_root,
        selections=[
            RepoBranchSelection(
                repo_key="__root__",
                source_repo_path=source_root,
                selected_ref="HEAD",
                selected_remote=None,
                relative_path=".",
            )
        ],
    )

    workspace_docs = tmp_path / "workspace" / "docs"
    assert workspace_docs.is_symlink()
    assert workspace_docs.resolve() == (source_root / "docs").resolve()
    assert (workspace_docs / "TASK_0079.md").read_text(encoding="utf-8") == "真实项目文档\n"


def test_prepare_parallel_workspace_creates_missing_source_docs_before_linking(tmp_path: Path) -> None:
    """真实项目缺少 docs 时，应先自动创建，再让工作区 docs 指向它。"""

    source_root = tmp_path / "source"
    _init_repo(source_root, {"README.md": "root\n"})

    prepare_parallel_workspace(
        workspace_root=tmp_path / "workspace",
        task_id="TASK_0080",
        title="自动创建 docs",
        source_root=source_root,
        selections=[
            RepoBranchSelection(
                repo_key="__root__",
                source_repo_path=source_root,
                selected_ref="HEAD",
                selected_remote=None,
                relative_path=".",
            )
        ],
    )

    source_docs = source_root / "docs"
    workspace_docs = tmp_path / "workspace" / "docs"
    assert source_docs.exists()
    assert source_docs.is_dir()
    assert workspace_docs.is_symlink()
    assert workspace_docs.resolve() == source_docs.resolve()


def test_prepare_parallel_workspace_generates_idea_vcs_mappings_for_nested_repos(tmp_path: Path) -> None:
    """并行工作区应补齐 IDEA 的多仓库 Git 映射，便于直接查看所有分支。"""

    source_root = tmp_path / "source"
    _init_repo(source_root, {"README.md": "root\n"})
    nested_repo = source_root / "service"
    _init_repo(nested_repo, {"src/main.py": "print('demo')\n"})

    prepare_parallel_workspace(
        workspace_root=tmp_path / "workspace",
        task_id="TASK_0107",
        title="生成 IDEA Git 映射",
        source_root=source_root,
        selections=[
            RepoBranchSelection(
                repo_key="__root__",
                source_repo_path=source_root,
                selected_ref="HEAD",
                selected_remote=None,
                relative_path=".",
            ),
            RepoBranchSelection(
                repo_key="service",
                source_repo_path=nested_repo,
                selected_ref="HEAD",
                selected_remote=None,
                relative_path="service",
            ),
        ],
    )

    vcs_xml = (tmp_path / "workspace" / ".idea" / "vcs.xml").read_text(encoding="utf-8")

    assert '<mapping directory="$PROJECT_DIR$" vcs="Git" />' in vcs_xml
    assert '<mapping directory="$PROJECT_DIR$/service" vcs="Git" />' in vcs_xml


def test_delete_parallel_workspace_keeps_shared_source_docs(tmp_path: Path) -> None:
    """删除并行工作区时，只能删除 docs 链接本身，不能删除真实项目 docs。"""

    source_docs = tmp_path / "source" / "docs"
    source_docs.mkdir(parents=True, exist_ok=True)
    (source_docs / "TASK_0079.md").write_text("保留\n", encoding="utf-8")

    workspace_root = tmp_path / "workspace"
    workspace_root.mkdir(parents=True, exist_ok=True)
    (workspace_root / "docs").symlink_to(source_docs, target_is_directory=True)

    delete_parallel_workspace(workspace_root=workspace_root)

    assert not workspace_root.exists()
    assert source_docs.exists()
    assert (source_docs / "TASK_0079.md").read_text(encoding="utf-8") == "保留\n"


def test_prepare_parallel_workspace_skips_fetch_for_local_branch_selection(
    tmp_path: Path,
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    """本地分支作为基线时，不应额外访问远端。"""

    source_root = tmp_path / "source"
    _init_repo(source_root, {"README.md": "root\n"})

    calls: list[tuple[tuple[str, ...], Path]] = []

    def fake_run_git(args: list[str], cwd: Path) -> subprocess.CompletedProcess[str]:
        calls.append((tuple(args), Path(cwd)))
        return subprocess.CompletedProcess(["git", *args], 0, stdout="", stderr="")

    monkeypatch.setattr(parallel_runtime, "_run_git", fake_run_git)

    records = prepare_parallel_workspace(
        workspace_root=tmp_path / "workspace",
        task_id="TASK_0100",
        title="本地分支跳过抓取",
        source_root=source_root,
        selections=[
            RepoBranchSelection(
                repo_key="__root__",
                source_repo_path=source_root,
                selected_ref="develop",
                selected_remote=None,
                relative_path=".",
            )
        ],
    )

    workspace_root = tmp_path / "workspace"
    fetch_calls = [args for args, cwd in calls if args[:3] == ("fetch", "--all", "--prune") and cwd == workspace_root]
    checkout_calls = [args for args, cwd in calls if args[:2] == ("checkout", "-B") and cwd == workspace_root]

    assert fetch_calls == []
    assert checkout_calls == [("checkout", "-B", records[0].task_branch, "develop")]


def test_prepare_parallel_workspace_fetches_only_selected_remote_repos(
    tmp_path: Path,
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    """仅选中远端分支的仓库需要抓取远端；本地分支仓库应直接检出。"""

    source_root = tmp_path / "source"
    _init_repo(source_root, {"README.md": "root\n"})
    nested_repo = source_root / "service"
    _init_repo(nested_repo, {"service.txt": "nested\n"})

    calls: list[tuple[tuple[str, ...], Path]] = []

    def fake_run_git(args: list[str], cwd: Path) -> subprocess.CompletedProcess[str]:
        calls.append((tuple(args), Path(cwd)))
        return subprocess.CompletedProcess(["git", *args], 0, stdout="", stderr="")

    monkeypatch.setattr(parallel_runtime, "_run_git", fake_run_git)

    records = prepare_parallel_workspace(
        workspace_root=tmp_path / "workspace",
        task_id="TASK_0101",
        title="混合本地远端基线",
        source_root=source_root,
        selections=[
            RepoBranchSelection(
                repo_key="__root__",
                source_repo_path=source_root,
                selected_ref="develop",
                selected_remote=None,
                relative_path=".",
            ),
            RepoBranchSelection(
                repo_key="service",
                source_repo_path=nested_repo,
                selected_ref="origin/develop",
                selected_remote="origin",
                relative_path="service",
            ),
        ],
    )

    workspace_root = tmp_path / "workspace"
    service_workspace = workspace_root / "service"
    root_fetch_calls = [args for args, cwd in calls if args[:3] == ("fetch", "--all", "--prune") and cwd == workspace_root]
    service_fetch_calls = [args for args, cwd in calls if args[:3] == ("fetch", "--all", "--prune") and cwd == service_workspace]
    root_checkout_calls = [args for args, cwd in calls if args[:2] == ("checkout", "-B") and cwd == workspace_root]
    service_checkout_calls = [args for args, cwd in calls if args[:2] == ("checkout", "-B") and cwd == service_workspace]

    assert root_fetch_calls == []
    assert service_fetch_calls == [("fetch", "--all", "--prune")]
    assert root_checkout_calls == [("checkout", "-B", records[0].task_branch, "develop")]
    assert service_checkout_calls == [("checkout", "-B", records[1].task_branch, "origin/develop")]
    assert calls.index((service_fetch_calls[0], service_workspace)) < calls.index((service_checkout_calls[0], service_workspace))


def test_prepare_parallel_workspace_preserves_dirty_changes_for_current_local_branch(tmp_path: Path) -> None:
    """当前分支创建并行任务分支时，应保留未提交改动进入并行目录。"""

    source_root = tmp_path / "source"
    _init_repo(source_root, {"README.md": "root\n"})
    current_branch = _current_branch(source_root)
    (source_root / "README.md").write_text("dirty root\n", encoding="utf-8")
    (source_root / "draft.txt").write_text("untracked\n", encoding="utf-8")

    records = prepare_parallel_workspace(
        workspace_root=tmp_path / "workspace",
        task_id="TASK_0102",
        title="保留当前分支未提交改动",
        source_root=source_root,
        selections=[
            RepoBranchSelection(
                repo_key="__root__",
                source_repo_path=source_root,
                selected_ref=current_branch,
                selected_remote=None,
                relative_path=".",
            )
        ],
    )

    workspace_root = tmp_path / "workspace"
    status = _git(workspace_root, "status", "--short", "--branch").stdout

    assert _current_branch(workspace_root) == records[0].task_branch
    assert (workspace_root / "README.md").read_text(encoding="utf-8") == "dirty root\n"
    assert (workspace_root / "draft.txt").read_text(encoding="utf-8") == "untracked\n"
    assert "README.md" in status
    assert "draft.txt" in status


def test_prepare_parallel_workspace_drops_dirty_changes_for_other_branch(tmp_path: Path) -> None:
    """切到非当前基线分支时，不应把当前工作区未提交改动带入并行目录。"""

    source_root = tmp_path / "source"
    _init_repo(source_root, {"README.md": "root\n"})
    current_branch = _current_branch(source_root)
    _git(source_root, "checkout", "-b", "develop")
    _git(source_root, "checkout", current_branch)
    (source_root / "README.md").write_text("dirty root\n", encoding="utf-8")
    (source_root / "draft.txt").write_text("untracked\n", encoding="utf-8")

    records = prepare_parallel_workspace(
        workspace_root=tmp_path / "workspace",
        task_id="TASK_0103",
        title="其他基线不带未提交改动",
        source_root=source_root,
        selections=[
            RepoBranchSelection(
                repo_key="__root__",
                source_repo_path=source_root,
                selected_ref="develop",
                selected_remote=None,
                relative_path=".",
            )
        ],
    )

    workspace_root = tmp_path / "workspace"
    status = _git(workspace_root, "status", "--short", "--branch").stdout

    assert _current_branch(workspace_root) == records[0].task_branch
    assert (workspace_root / "README.md").read_text(encoding="utf-8") == "root\n"
    assert not (workspace_root / "draft.txt").exists()
    assert status.strip() == f"## {records[0].task_branch}"


def test_commit_parallel_repos_skips_meta_only_parent_and_keeps_real_child_commit(tmp_path: Path) -> None:
    """父仓库仅包含嵌套仓库指针变化时，应跳过空壳提交；真实子仓库仍正常本地提交。"""

    root = tmp_path / "workspace"
    _init_repo(root, {"README.md": "root\n"})
    child = root / "service"
    _init_repo(child, {"service.py": "print('v1')\n"})

    _git(root, "add", "service")
    _git(root, "commit", "-m", "track child")

    task_branch = "TASK_0104-meta"
    _git(root, "checkout", "-b", task_branch)
    _git(child, "checkout", "-b", task_branch)
    (child / "service.py").write_text("print('v2')\n", encoding="utf-8")

    result = commit_parallel_repos(
        task=_make_task(task_id="TASK_0104", title="跳过空壳父仓库提交"),
        repos=[
            parallel_runtime.ParallelRepoRecord(
                repo_key="__root__",
                source_repo_path=str(root),
                workspace_repo_path=str(root),
                selected_base_ref=_current_branch(root),
                selected_remote=None,
                task_branch=task_branch,
            ),
            parallel_runtime.ParallelRepoRecord(
                repo_key="service",
                source_repo_path=str(child),
                workspace_repo_path=str(child),
                selected_base_ref=_current_branch(child),
                selected_remote=None,
                task_branch=task_branch,
            ),
        ],
    )

    by_repo = {item.repo_key: item for item in result.results}

    assert by_repo["__root__"].ok is True
    assert by_repo["__root__"].status == "skipped"
    assert "元信息" in by_repo["__root__"].message
    assert by_repo["service"].ok is True
    assert by_repo["service"].status == "committed"
    assert "本地提交" in by_repo["service"].message
    assert _git(child, "log", "--oneline", "-n", "1").stdout.split()[1].startswith("chore(TASK_0104):")


def test_commit_parallel_repos_pushes_to_single_available_remote(tmp_path: Path) -> None:
    """未显式记录 selected_remote 时，若仓库仅有一个 remote，仍应正常推送任务分支。"""

    remote = tmp_path / "remote.git"
    _init_bare_remote(remote)

    repo = tmp_path / "repo"
    _init_repo(repo, {"README.md": "demo\n"})
    base_branch = _current_branch(repo)
    _git(repo, "remote", "add", "origin", str(remote))
    _git(repo, "push", "-u", "origin", base_branch)
    task_branch = "TASK_0105-push"
    _git(repo, "checkout", "-b", task_branch)
    (repo / "README.md").write_text("demo v2\n", encoding="utf-8")

    result = commit_parallel_repos(
        task=_make_task(task_id="TASK_0105", title="唯一远端推送"),
        repos=[
            parallel_runtime.ParallelRepoRecord(
                repo_key="repo",
                source_repo_path=str(repo),
                workspace_repo_path=str(repo),
                selected_base_ref=base_branch,
                selected_remote=None,
                task_branch=task_branch,
            )
        ],
    )

    assert result.results[0].ok is True
    assert result.results[0].status == "pushed"
    remote_refs = _git(remote, "for-each-ref", "--format=%(refname:short)", "refs/heads").stdout.splitlines()
    assert task_branch in remote_refs


def test_merge_parallel_repos_skips_repo_without_remote(tmp_path: Path) -> None:
    """无远端仓库在自动合并阶段应跳过，而不是默认 fetch/push origin 失败。"""

    repo = tmp_path / "repo"
    _init_repo(repo, {"README.md": "demo\n"})
    base_branch = _current_branch(repo)
    task_branch = "TASK_0106-merge"
    _git(repo, "checkout", "-b", task_branch)
    (repo / "README.md").write_text("demo v2\n", encoding="utf-8")
    _git(repo, "add", "README.md")
    _git(repo, "commit", "-m", "task change")

    result = merge_parallel_repos(
        task=_make_task(task_id="TASK_0106", title="无远端跳过自动合并"),
        repos=[
            parallel_runtime.ParallelRepoRecord(
                repo_key="repo",
                source_repo_path=str(repo),
                workspace_repo_path=str(repo),
                selected_base_ref=base_branch,
                selected_remote=None,
                task_branch=task_branch,
            )
        ],
    )

    assert result.results[0].ok is True
    assert result.results[0].status == "skipped"
    assert "远端" in result.results[0].message


def test_get_current_branch_state_marks_current_local_branch_first(tmp_path: Path) -> None:
    """应识别当前本地分支，并在分支列表中置顶且标记为当前。"""

    repo = tmp_path / "repo"
    _init_repo(repo, {"README.md": "demo\n"})
    _git(repo, "checkout", "-b", "develop")
    _git(repo, "branch", "feature/demo")

    current_label, current_local_branch = get_current_branch_state(repo)
    branches = list_branch_refs(repo, current_local_branch=current_local_branch)

    assert current_label == "develop"
    assert current_local_branch == "develop"
    assert branches[0].name == "develop"
    assert branches[0].is_current is True


def test_get_current_branch_state_returns_detached_head_when_not_on_branch(tmp_path: Path) -> None:
    """Detached HEAD 应展示专用文案，且不标记任何当前分支。"""

    repo = tmp_path / "repo"
    _init_repo(repo, {"README.md": "demo\n"})
    head_sha = _git(repo, "rev-parse", "HEAD").stdout.strip()
    _git(repo, "checkout", "--detach", head_sha)

    current_label, current_local_branch = get_current_branch_state(repo)
    branches = list_branch_refs(repo, current_local_branch=current_local_branch)

    assert current_label == "Detached HEAD"
    assert current_local_branch is None
    assert all(branch.is_current is False for branch in branches)


def test_get_current_branch_state_returns_read_failure_for_non_repo(tmp_path: Path) -> None:
    """非 Git 目录读取当前分支失败时，应返回统一兜底文案。"""

    plain_dir = tmp_path / "plain"
    plain_dir.mkdir(parents=True, exist_ok=True)

    current_label, current_local_branch = get_current_branch_state(plain_dir)

    assert current_label == "读取失败"
    assert current_local_branch is None



from pathlib import Path

from bot import (
    WX_UPLOAD_COMMAND_NAME,
    _apply_command_env_overrides,
    _extract_command_args,
    _extract_command_trigger,
    _parse_wx_upload_args,
)
from command_center import CommandDefinition


def test_extract_command_trigger_and_args() -> None:
    prompt = "/wx-dev-upload --version 202602140001"
    assert _extract_command_trigger(prompt) == "wx-dev-upload"
    assert _extract_command_args(prompt) == "--version 202602140001"


def test_parse_wx_upload_args_empty() -> None:
    version, err = _parse_wx_upload_args("")
    assert version is None
    assert err is None


def test_parse_wx_upload_args_version() -> None:
    version, err = _parse_wx_upload_args("--version 202602140001")
    assert err is None
    assert version == "202602140001"


def test_parse_wx_upload_args_rejects_unknown_option() -> None:
    version, err = _parse_wx_upload_args("--robot 1")
    assert version is None
    assert err is not None


def test_parse_wx_upload_args_rejects_invalid_version() -> None:
    version, err = _parse_wx_upload_args("--version bad/version")
    assert version is None
    assert err is not None


def test_apply_command_env_overrides_injects_version(tmp_path: Path) -> None:
    command = CommandDefinition(
        id=1,
        project_slug="demo",
        name=WX_UPLOAD_COMMAND_NAME,
        title="上传",
        command='PROJECT_BASE="${PROJECT_BASE:-$MODEL_WORKDIR}" bash "$ROOT_DIR/scripts/gen_upload.sh"',
        scope="global",
        description="",
        timeout=600,
        enabled=True,
        aliases=(),
    )
    wrapped = _apply_command_env_overrides(command, {"VERSION": "202602140001"})
    assert wrapped.command.startswith("VERSION=202602140001 ")
    assert "gen_upload.sh" in wrapped.command



from __future__ import annotations

import asyncio
import os
import sys
from datetime import datetime, timezone
from pathlib import Path
from typing import Optional

import pytest
from aiogram.types import Chat, Message, User

ROOT = Path(__file__).resolve().parents[1]
if str(ROOT) not in sys.path:
    sys.path.insert(0, str(ROOT))

os.environ.setdefault("BOT_TOKEN", "TEST_TOKEN")

import bot


def _utc_now() -> datetime:
    try:
        from datetime import UTC  # type: ignore

        return datetime.now(UTC)
    except Exception:  # pragma: no cover - Python <3.11 兜底
        return datetime.now(timezone.utc)


def _build_message(*, chat_id: int, message_id: int, text: Optional[str] = "hello") -> Message:
    chat = Chat.model_construct(id=chat_id, type="private")
    user = User.model_construct(id=123, is_bot=False, first_name="Tester")
    return Message.model_construct(
        message_id=message_id,
        date=_utc_now(),
        chat=chat,
        text=text,
        from_user=user,
    )


@pytest.fixture(autouse=True)
def _reset_global_state():
    bot.CHAT_MESSAGE_RECOVERY_POLL_TASKS.clear()
    bot.CHAT_SESSION_MAP.clear()
    bot.SESSION_OFFSETS.clear()
    bot.CHAT_DELIVERED_OFFSETS.clear()
    bot.CHAT_LAST_MESSAGE.clear()
    bot.CHAT_WATCHERS.clear()
    bot.TEXT_PASTE_SYNTHETIC_GUARD.clear()
    yield
    bot.CHAT_MESSAGE_RECOVERY_POLL_TASKS.clear()
    bot.CHAT_SESSION_MAP.clear()
    bot.SESSION_OFFSETS.clear()
    bot.CHAT_DELIVERED_OFFSETS.clear()
    bot.CHAT_LAST_MESSAGE.clear()
    bot.CHAT_WATCHERS.clear()
    bot.TEXT_PASTE_SYNTHETIC_GUARD.clear()


@pytest.mark.asyncio
async def test_schedule_message_recovery_poll_stops_after_first_hit(monkeypatch):
    """命中新消息后，应提前结束后续轮次。"""

    chat_id = 2001
    message = _build_message(chat_id=chat_id, message_id=11, text="ping")
    monkeypatch.setattr(bot, "MESSAGE_RECOVERY_POLL_DELAYS_SECONDS", (0.01, 0.01, 0.01))

    rounds: list[int] = []

    async def fake_probe(*_args, round_index: int, **_kwargs) -> bool:
        rounds.append(round_index)
        return round_index == 1

    monkeypatch.setattr(bot, "_probe_new_model_message_once", fake_probe)

    await bot._schedule_message_recovery_poll(message, source="test")
    await asyncio.sleep(0.08)

    assert rounds == [1]
    assert chat_id not in bot.CHAT_MESSAGE_RECOVERY_POLL_TASKS


@pytest.mark.asyncio
async def test_schedule_message_recovery_poll_overrides_previous_task(monkeypatch):
    """同一 chat 新消息应覆盖旧补偿任务。"""

    chat_id = 2002
    msg1 = _build_message(chat_id=chat_id, message_id=21, text="first")
    msg2 = _build_message(chat_id=chat_id, message_id=22, text="second")
    monkeypatch.setattr(bot, "MESSAGE_RECOVERY_POLL_DELAYS_SECONDS", (0.05,))

    trigger_ids: list[int] = []

    async def fake_probe(*_args, trigger_message_id: int, **_kwargs) -> bool:
        trigger_ids.append(trigger_message_id)
        return False

    monkeypatch.setattr(bot, "_probe_new_model_message_once", fake_probe)

    await bot._schedule_message_recovery_poll(msg1, source="test")
    first_task = bot.CHAT_MESSAGE_RECOVERY_POLL_TASKS[chat_id]
    await asyncio.sleep(0.005)

    await bot._schedule_message_recovery_poll(msg2, source="test")
    second_task = bot.CHAT_MESSAGE_RECOVERY_POLL_TASKS[chat_id]

    assert first_task is not second_task
    assert first_task.done()

    await asyncio.sleep(0.08)
    assert trigger_ids == [22]
    assert chat_id not in bot.CHAT_MESSAGE_RECOVERY_POLL_TASKS


@pytest.mark.asyncio
async def test_schedule_message_recovery_poll_skips_internal_synthetic_message():
    """内部合成消息不应触发补偿轮询。"""

    chat_id = 2003
    message = _build_message(chat_id=chat_id, message_id=31, text="/task_new")
    bot._mark_text_paste_synthetic_message(chat_id, message.message_id)

    await bot._schedule_message_recovery_poll(message, source="test")

    assert chat_id not in bot.CHAT_MESSAGE_RECOVERY_POLL_TASKS


def test_dispatch_prompt_keeps_watcher_after_quick_delivery(monkeypatch, tmp_path: Path):
    """即时轮询已命中输出时，仍应补建 watcher 监听后续消息。"""

    pointer = tmp_path / "pointer.txt"
    session_file = tmp_path / "rollout.jsonl"
    session_file.write_text("", encoding="utf-8")
    pointer.write_text(str(session_file), encoding="utf-8")

    monkeypatch.setattr(bot, "CODEX_SESSION_FILE_PATH", str(pointer))
    monkeypatch.setattr(bot, "CODEX_WORKDIR", "")
    monkeypatch.setattr(bot, "SESSION_BIND_STRICT", True)
    monkeypatch.setattr(bot, "SESSION_POLL_TIMEOUT", 0.2)

    def fake_tmux_send_line(_session: str, _line: str) -> None:
        return

    monkeypatch.setattr(bot, "tmux_send_line", fake_tmux_send_line)
    monkeypatch.setattr(bot, "_interrupt_long_poll", lambda _chat_id: asyncio.sleep(0))

    delivered_calls = {"count": 0}

    async def fake_deliver(_chat_id: int, _path: Path, **_kwargs) -> bool:
        delivered_calls["count"] += 1
        return delivered_calls["count"] == 1

    monkeypatch.setattr(bot, "_deliver_pending_messages", fake_deliver)

    watch_start_flags: list[bool] = []

    def fake_watch(_chat_id: int, _path: Path, max_wait: float, interval: float, *, start_in_long_poll: bool = False):
        watch_start_flags.append(start_in_long_poll)

        async def _noop():
            return None

        return _noop()

    monkeypatch.setattr(bot, "_watch_and_notify", fake_watch)

    created_coroutines: list = []

    class DummyTask:
        def __init__(self) -> None:
            self._done = False

        def done(self) -> bool:
            return self._done

        def cancel(self) -> None:
            self._done = True

    def fake_create_task(coro):
        created_coroutines.append(coro)
        return DummyTask()

    monkeypatch.setattr(asyncio, "create_task", fake_create_task)

    async def _scenario() -> None:
        ok, path = await bot._dispatch_prompt_to_model(3001, "pwd", reply_to=None, ack_immediately=False)
        assert ok
        assert path == session_file

    asyncio.run(_scenario())

    assert watch_start_flags == [True]
    for coro in created_coroutines:
        try:
            coro.close()  # type: ignore[attr-defined]
        except Exception:  # pragma: no cover - 最佳努力清理
            pass


def test_ensure_session_watcher_keeps_watcher_after_quick_delivery(monkeypatch, tmp_path: Path):
    """ensure_session_watcher 即时命中 pending 后，仍应继续补建 watcher。"""

    pointer = tmp_path / "pointer.txt"
    session_file = tmp_path / "rollout.jsonl"
    session_file.write_text("", encoding="utf-8")
    pointer.write_text(str(session_file), encoding="utf-8")

    monkeypatch.setattr(bot, "CODEX_SESSION_FILE_PATH", str(pointer))
    monkeypatch.setattr(bot, "CODEX_WORKDIR", "")
    monkeypatch.setattr(bot, "SESSION_BIND_STRICT", True)

    async def fake_deliver(_chat_id: int, _path: Path, **_kwargs) -> bool:
        return True

    monkeypatch.setattr(bot, "_deliver_pending_messages", fake_deliver)
    monkeypatch.setattr(bot, "_interrupt_long_poll", lambda _chat_id: asyncio.sleep(0))

    watch_start_flags: list[bool] = []

    def fake_watch(_chat_id: int, _path: Path, max_wait: float, interval: float, *, start_in_long_poll: bool = False):
        watch_start_flags.append(start_in_long_poll)

        async def _noop():
            return None

        return _noop()

    monkeypatch.setattr(bot, "_watch_and_notify", fake_watch)

    created_coroutines: list = []

    class DummyTask:
        def __init__(self) -> None:
            self._done = False

        def done(self) -> bool:
            return self._done

        def cancel(self) -> None:
            self._done = True

    def fake_create_task(coro):
        created_coroutines.append(coro)
        return DummyTask()

    monkeypatch.setattr(asyncio, "create_task", fake_create_task)

    result = asyncio.run(bot._ensure_session_watcher(3002))

    assert result == session_file
    assert watch_start_flags == [True]
    assert isinstance(bot.CHAT_WATCHERS[3002], DummyTask)

    for coro in created_coroutines:
        try:
            coro.close()  # type: ignore[attr-defined]
        except Exception:  # pragma: no cover - 最佳努力清理
            pass



"""存放 CLI runtime 所需的数据文件（例如 worker requirements）。"""

from importlib import resources


def load_worker_requirements() -> str:
    """读取 worker 依赖清单，供 CLI 初始化虚拟环境时使用。"""

    with resources.files(__package__).joinpath("worker_requirements.txt").open("r", encoding="utf-8") as fp:
        return fp.read()



import sys
import os
from pathlib import Path

sys.path.append(str(Path(__file__).resolve().parents[1]))

import pytest

os.environ.setdefault("BOT_TOKEN", "dummy")
os.environ.setdefault("TELEGRAM_PARSE_MODE", "Markdown")

from bot import _prepare_model_payload, _prepare_model_payload_variants  # noqa: E402


@pytest.mark.parametrize(
    ("source", "expected"),
    [
        ("**bold** text", "*bold* text"),
        ("__italic__", "_italic_"),
        ("`code_snippet`", "`code_snippet`"),
        ("普通文本 (test)", "普通文本 (test)"),
        ("raw *literal* star", "raw *literal* star"),
        ("multiline\nline2", "multiline\nline2"),
    ],
)
def test_prepare_model_payload_markdown(source: str, expected: str) -> None:
    assert _prepare_model_payload(source) == expected


def test_prepare_model_payload_preserves_checklist() -> None:
    text = "- [ ] 第一项\n- [ ] 第二项"
    assert _prepare_model_payload(text) == text


def test_prepare_model_payload_handles_list_numbers() -> None:
    text = "1. 选择 A\n2. 选择 B"
    assert _prepare_model_payload(text) == text


def test_prepare_model_payload_variants_returns_single_payload() -> None:
    text = "纯文本内容"
    primary, fallback = _prepare_model_payload_variants(text)
    assert primary == "纯文本内容"
    assert fallback is None



from __future__ import annotations

import argparse
import importlib
from pathlib import Path

import pytest

config = importlib.import_module("vibego_cli.config")
cli_main = importlib.import_module("vibego_cli.main")


class _FakeProcess:
    """模拟 master 子进程，避免测试真实拉起后台进程。"""

    def __init__(self, *, pid: int, poll_result: int | None) -> None:
        self.pid = pid
        self._poll_result = poll_result

    def poll(self) -> int | None:
        """返回预设轮询结果，模拟进程存活或已退出。"""

        return self._poll_result


def _patch_start_prerequisites(monkeypatch: pytest.MonkeyPatch, tmp_path: Path) -> None:
    """统一打桩 start 前置依赖，确保测试仅聚焦启动收口逻辑。"""

    monkeypatch.setattr(config, "MASTER_PID_FILE", tmp_path / "master.pid")
    monkeypatch.setattr(config, "LOG_FILE", tmp_path / "vibe.log")
    monkeypatch.setattr(cli_main, "_load_env_or_fail", lambda: {"MASTER_BOT_TOKEN": "token"})
    monkeypatch.setattr(cli_main, "_ensure_projects_assets", lambda: None)
    monkeypatch.setattr(cli_main, "_find_foreign_processes", lambda _root: [])
    monkeypatch.setattr(cli_main, "_collect_active_workers", lambda: [])
    monkeypatch.setattr(cli_main, "python_version_ok", lambda: True)
    monkeypatch.setattr(cli_main, "check_cli_dependencies", lambda: [])
    monkeypatch.setattr(cli_main, "_find_repo_root", lambda: tmp_path)
    monkeypatch.setattr(
        cli_main,
        "_ensure_virtualenv",
        lambda _root: (tmp_path / "python", tmp_path / "pip"),
    )
    monkeypatch.setattr(cli_main, "_build_master_env", lambda _env: {})
    monkeypatch.setattr(cli_main, "_schedule_start_notification", lambda _env: False)
    monkeypatch.setattr(cli_main.time, "sleep", lambda _seconds: None)


def test_command_start_clears_stale_pid_file_when_master_exits_early(
    monkeypatch: pytest.MonkeyPatch,
    tmp_path: Path,
) -> None:
    """master 启动即退出时，应清理刚写入的 pid，避免后续误判已启动。"""

    _patch_start_prerequisites(monkeypatch, tmp_path)

    monkeypatch.setattr(
        cli_main.subprocess,
        "Popen",
        lambda *_args, **_kwargs: _FakeProcess(pid=4321, poll_result=1),
    )

    with pytest.raises(RuntimeError, match="master 进程启动失败"):
        cli_main.command_start(argparse.Namespace())

    assert not config.MASTER_PID_FILE.exists()


def test_command_start_ignores_dead_master_pid_before_launch(
    monkeypatch: pytest.MonkeyPatch,
    tmp_path: Path,
) -> None:
    """若 pid 文件残留但进程已死，start 应自动清理并继续拉起新 master。"""

    _patch_start_prerequisites(monkeypatch, tmp_path)
    config.MASTER_PID_FILE.write_text("99999", encoding="utf-8")
    monkeypatch.setattr(cli_main, "_pid_alive", lambda pid: pid == 4321)

    calls = {"popen": 0}

    def fake_popen(*_args, **_kwargs):
        calls["popen"] += 1
        return _FakeProcess(pid=4321, poll_result=None)

    monkeypatch.setattr(cli_main.subprocess, "Popen", fake_popen)

    cli_main.command_start(argparse.Namespace())

    assert calls["popen"] == 1
    assert config.MASTER_PID_FILE.read_text(encoding="utf-8") == "4321"



from __future__ import annotations

import asyncio
import os
import sys
import time
from pathlib import Path
from types import SimpleNamespace

from aiogram.fsm.context import FSMContext
from aiogram.fsm.storage.base import StorageKey
from aiogram.fsm.storage.memory import MemoryStorage

ROOT = Path(__file__).resolve().parents[1]
if str(ROOT) not in sys.path:
    sys.path.insert(0, str(ROOT))

os.environ.setdefault("BOT_TOKEN", "TEST_TOKEN")

import bot  # noqa: E402


class DummyMessage:
    """模拟最小可用的 Telegram Message。"""

    def __init__(self, *, chat_id: int = 1, user_id: int = 1, text: str | None = None):
        self.calls: list[tuple[str, object, object, dict]] = []
        self.chat = SimpleNamespace(id=chat_id)
        self.from_user = SimpleNamespace(id=user_id, full_name="Tester")
        self.message_id = 100
        self.text = text
        self.caption = None

    async def answer(self, text: str, parse_mode=None, reply_markup=None, **kwargs):
        self.calls.append((text, parse_mode, reply_markup, kwargs))
        return SimpleNamespace(message_id=self.message_id + len(self.calls), chat=self.chat)


class DummyCallback:
    """模拟最小可用的 Telegram CallbackQuery。"""

    def __init__(self, data: str, message: DummyMessage):
        self.data = data
        self.message = message
        self.answers: list[tuple[str | None, bool]] = []
        self.from_user = SimpleNamespace(id=message.from_user.id, full_name="Tester")

    async def answer(self, text: str | None = None, show_alert: bool = False):
        self.answers.append((text, show_alert))


def _make_state(message: DummyMessage) -> tuple[FSMContext, MemoryStorage]:
    storage = MemoryStorage()
    state = FSMContext(
        storage=storage,
        key=StorageKey(bot_id=999, chat_id=message.chat.id, user_id=message.from_user.id),
    )
    return state, storage


def _parallel_context(task_id: str = "TASK_0093") -> bot.ParallelDispatchContext:
    return bot.ParallelDispatchContext(
        task_id=task_id,
        tmux_session="vibe-par-demo",
        pointer_file=Path("/tmp/demo-pointer.txt"),
        workspace_root=Path("/tmp/demo-workspace"),
    )


def test_parallel_keyboard_uses_session_scoped_callback_payload():
    """并行消息底部按钮应携带会话级 payload，避免仅靠 task_id 串到原生会话。"""

    markup = bot._build_model_quick_reply_keyboard(
        task_id="TASK_0093",
        parallel_task_title="并行任务",
        enable_parallel_actions=True,
        parallel_callback_payload="TASK_0093:deadbeef",
    )
    callback_data = [
        button.callback_data
        for row in markup.inline_keyboard
        for button in row
        if button.callback_data
    ]

    assert f"{bot.MODEL_QUICK_REPLY_ALL_TASK_PREFIX}TASK_0093:deadbeef" in callback_data
    assert f"{bot.MODEL_QUICK_REPLY_PARTIAL_TASK_PREFIX}TASK_0093:deadbeef" in callback_data
    assert f"{bot.PARALLEL_REPLY_CALLBACK_PREFIX}TASK_0093:deadbeef" in callback_data


def test_quick_reply_all_uses_parallel_binding_dispatch_context(monkeypatch, tmp_path: Path):
    """并行消息上的“全部按推荐”应严格命中并行上下文，不能回落到原生会话。"""

    origin = DummyMessage(chat_id=9, user_id=9)
    callback = DummyCallback(f"{bot.MODEL_QUICK_REPLY_ALL_TASK_PREFIX}TASK_0093:deadbeef", origin)
    binding = SimpleNamespace(
        token="deadbeef",
        task_id="TASK_0093",
        dispatch_context=_parallel_context(),
    )
    bot.PARALLEL_CALLBACK_BINDINGS = {"deadbeef": binding}

    recorded: list[object] = []

    async def fake_dispatch(chat_id: int, prompt: str, *, reply_to, ack_immediately: bool = True, dispatch_context=None, **_kwargs):
        recorded.append(dispatch_context)
        return True, tmp_path / "parallel.jsonl"

    async def fake_preview(*_args, **_kwargs):
        return None

    async def fake_ack(*_args, **_kwargs):
        return None

    async def fake_active_parallel_session(_task_id: str):
        return None

    monkeypatch.setattr(bot, "_dispatch_prompt_to_model", fake_dispatch)
    monkeypatch.setattr(bot, "_send_model_push_preview", fake_preview)
    monkeypatch.setattr(bot, "_send_session_ack", fake_ack)
    monkeypatch.setattr(bot, "_get_active_parallel_session_for_task", fake_active_parallel_session)

    asyncio.run(bot.on_model_quick_reply_all(callback))

    assert recorded and recorded[-1] is binding.dispatch_context


def test_quick_reply_all_parallel_payload_fails_closed_when_binding_missing(monkeypatch):
    """并行按钮若已失效，必须直接报错，不能静默回落到原生会话。"""

    origin = DummyMessage(chat_id=10, user_id=10)
    callback = DummyCallback(f"{bot.MODEL_QUICK_REPLY_ALL_TASK_PREFIX}TASK_0093:deadbeef", origin)
    bot.PARALLEL_CALLBACK_BINDINGS = {}

    async def fake_dispatch(*_args, **_kwargs):
        raise AssertionError("并行会话失效时不应继续回落到默认会话")

    async def fake_active_parallel_session(_task_id: str):
        return None

    monkeypatch.setattr(bot, "_dispatch_prompt_to_model", fake_dispatch)
    monkeypatch.setattr(bot, "_get_active_parallel_session_for_task", fake_active_parallel_session)

    asyncio.run(bot.on_model_quick_reply_all(callback))

    assert callback.answers and callback.answers[-1] == ("并行会话已失效，请在最新并行消息中重试。", True)
    assert origin.calls and "并行会话已失效" in origin.calls[-1][0]


def test_parallel_reply_callback_and_followup_use_bound_dispatch_context(monkeypatch):
    """回复按钮应优先使用按钮绑定的并行上下文，而不是再次依赖活动会话查库。"""

    origin = DummyMessage(chat_id=12, user_id=12)
    callback = DummyCallback(f"{bot.PARALLEL_REPLY_CALLBACK_PREFIX}TASK_0093:deadbeef", origin)
    binding = SimpleNamespace(
        token="deadbeef",
        task_id="TASK_0093",
        dispatch_context=_parallel_context(),
    )
    bot.PARALLEL_CALLBACK_BINDINGS = {"deadbeef": binding}

    async def fake_active_parallel_session(_task_id: str):
        return None

    recorded: list[tuple[str, object]] = []

    async def fake_handle_request_input_custom_text_message(_message):
        return False

    async def fake_handle_command_trigger_message(_message, _prompt, _state):
        return False

    async def fake_handle_prompt_dispatch(_message, prompt: str, *, dispatch_context=None):
        recorded.append((prompt, dispatch_context))

    monkeypatch.setattr(bot, "_get_active_parallel_session_for_task", fake_active_parallel_session)
    monkeypatch.setattr(bot, "_handle_request_input_custom_text_message", fake_handle_request_input_custom_text_message)
    monkeypatch.setattr(bot, "_handle_command_trigger_message", fake_handle_command_trigger_message)
    monkeypatch.setattr(bot, "_handle_prompt_dispatch", fake_handle_prompt_dispatch)

    async def _scenario() -> None:
        await bot.on_parallel_reply_callback(callback)
        message = DummyMessage(chat_id=12, user_id=12, text="继续补充")
        state, _ = _make_state(message)
        await bot.on_text(message, state)

    asyncio.run(_scenario())

    assert recorded == [("继续补充", binding.dispatch_context)]


def test_manual_task_prefix_routes_parallel_session_without_leaking_slash_prefix(monkeypatch):
    """手动输入 /TASK_xxxx 仅用于路由，真正入模文本不应保留 slash 前缀。"""

    message = DummyMessage(chat_id=15, user_id=15, text="/TASK_0093 继续补充")
    state, _ = _make_state(message)
    binding = SimpleNamespace(
        token="deadbeef",
        task_id="TASK_0093",
        dispatch_context=_parallel_context(),
    )
    bot.PARALLEL_CALLBACK_BINDINGS = {"deadbeef": binding}

    async def fake_active_parallel_session(_task_id: str):
        return SimpleNamespace(
            task_id="TASK_0093",
            tmux_session=binding.dispatch_context.tmux_session,
            pointer_file=str(binding.dispatch_context.pointer_file),
            workspace_root=str(binding.dispatch_context.workspace_root),
        )

    recorded: list[tuple[str, object]] = []

    async def fake_handle_request_input_custom_text_message(_message):
        return False

    async def fake_handle_command_trigger_message(_message, _prompt, _state):
        return False

    async def fake_handle_prompt_dispatch(_message, prompt: str, *, dispatch_context=None):
        recorded.append((prompt, dispatch_context))

    monkeypatch.setattr(bot, "_get_active_parallel_session_for_task", fake_active_parallel_session)
    monkeypatch.setattr(bot, "_handle_request_input_custom_text_message", fake_handle_request_input_custom_text_message)
    monkeypatch.setattr(bot, "_handle_command_trigger_message", fake_handle_command_trigger_message)
    monkeypatch.setattr(bot, "_handle_prompt_dispatch", fake_handle_prompt_dispatch)

    asyncio.run(bot.on_text(message, state))

    assert recorded == [("继续补充", binding.dispatch_context)]


def test_parallel_reply_cancel_restores_main_keyboard(monkeypatch):
    """回复态发送“取消”时应退出回复模式并恢复主菜单。"""

    message = DummyMessage(chat_id=18, user_id=18, text="取消")
    state, _ = _make_state(message)
    bot._set_parallel_reply_target(
        message.chat.id,
        "TASK_0115",
        dispatch_context=_parallel_context("TASK_0115"),
    )

    async def fake_handle_request_input_custom_text_message(_message):
        return False

    monkeypatch.setattr(bot, "_handle_request_input_custom_text_message", fake_handle_request_input_custom_text_message)

    asyncio.run(bot.on_text(message, state))

    assert message.calls and message.calls[-1][0] == "已取消 /TASK_0115 回复模式。"
    reply_markup = message.calls[-1][2]
    labels = [button.text for row in reply_markup.keyboard for button in row]
    assert bot.WORKER_MENU_BUTTON_TEXT in labels
    assert message.chat.id not in bot.CHAT_PARALLEL_REPLY_TARGETS


def test_parallel_dispatch_does_not_override_primary_session_binding(monkeypatch, tmp_path: Path):
    """向并行会话发消息时，不应覆盖 chat 当前原生会话，也不应取消原生 watcher。"""

    primary_session = tmp_path / "primary.jsonl"
    parallel_session = tmp_path / "parallel.jsonl"
    pointer = tmp_path / "parallel-pointer.txt"
    primary_session.write_text("", encoding="utf-8")
    parallel_session.write_text("", encoding="utf-8")
    pointer.write_text(str(parallel_session), encoding="utf-8")

    chat_id = 77
    bot.CHAT_SESSION_MAP[chat_id] = str(primary_session)

    class ActiveTask:
        def __init__(self) -> None:
            self.cancelled = False

        def done(self) -> bool:
            return False

        def cancel(self) -> None:
            self.cancelled = True

    active_watcher = ActiveTask()
    bot.CHAT_WATCHERS[chat_id] = active_watcher

    def fake_tmux_send_line(_session: str, _prompt: str) -> None:
        return None

    async def fake_deliver(_chat_id: int, _session_path: Path, **_kwargs) -> bool:
        return False

    async def fake_interrupt_long_poll(_chat_id: int) -> None:
        return None

    async def fake_parallel_watch(*_args, **_kwargs):
        return None

    created_tasks: list[object] = []

    class DummyTask:
        def done(self) -> bool:
            return False

        def cancel(self) -> None:
            return None

    def fake_create_task(coro):
        created_tasks.append(coro)
        return DummyTask()

    monkeypatch.setattr(bot, "tmux_send_line", fake_tmux_send_line)
    monkeypatch.setattr(bot, "_deliver_pending_messages", fake_deliver)
    monkeypatch.setattr(bot, "_interrupt_long_poll", fake_interrupt_long_poll)
    monkeypatch.setattr(bot, "_watch_parallel_and_notify", fake_parallel_watch)
    monkeypatch.setattr(asyncio, "create_task", fake_create_task)

    async def _scenario() -> None:
        ok, session_path = await bot._dispatch_prompt_to_model(
            chat_id,
            "待决策项全部按模型推荐",
            reply_to=None,
            ack_immediately=False,
            dispatch_context=bot.ParallelDispatchContext(
                task_id="TASK_0093",
                tmux_session="vibe-par-demo",
                pointer_file=pointer,
                workspace_root=tmp_path / "workspace",
            ),
        )
        assert ok is True
        assert session_path == parallel_session

    asyncio.run(_scenario())

    assert bot.CHAT_SESSION_MAP[chat_id] == str(primary_session)
    assert active_watcher.cancelled is False
    for coro in created_tasks:
        try:
            coro.close()  # type: ignore[attr-defined]
        except Exception:
            pass


def test_get_active_parallel_session_marks_stale_session_closed(monkeypatch):
    """活动并行会话若 tmux 已消失，应自动降级为 closed 并清理内存绑定。"""

    task_id = "TASK_0115"
    session = SimpleNamespace(
        task_id=task_id,
        status="running",
        tmux_session="vibe-par-hyphamall-task_0115",
        pointer_file="/tmp/parallel-pointer.txt",
        workspace_root="/tmp/parallel-workspace",
    )

    updates: list[tuple[str, dict]] = []

    async def fake_get_session(_task_id: str):
        return session

    async def fake_update_status(_task_id: str, **kwargs):
        updates.append((_task_id, kwargs))

    watcher_cancelled = {"value": False}

    class DummyTask:
        def done(self) -> bool:
            return False

        def cancel(self) -> None:
            watcher_cancelled["value"] = True

    bot.PARALLEL_TASK_SESSION_MAP[task_id] = "session-key"
    bot.PARALLEL_SESSION_CONTEXTS["session-key"] = _parallel_context(task_id)
    bot.PARALLEL_CALLBACK_BINDINGS = {"deadbeef": SimpleNamespace(task_id=task_id)}
    bot.PARALLEL_TASK_WATCHERS[task_id] = DummyTask()

    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "get_session", fake_get_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "update_status", fake_update_status)
    monkeypatch.setattr(
        bot,
        "_parallel_session_runtime_issue",
        lambda _session: "tmux 会话 vibe-par-hyphamall-task_0115 不存在",
    )

    result = asyncio.run(bot._get_active_parallel_session_for_task(task_id))

    assert result is None
    assert updates and updates[-1][0] == task_id
    assert updates[-1][1]["status"] == "closed"
    assert "tmux 会话" in updates[-1][1]["last_error"]
    assert task_id not in bot.PARALLEL_TASK_SESSION_MAP
    assert "session-key" not in bot.PARALLEL_SESSION_CONTEXTS
    assert watcher_cancelled["value"] is True
    assert not bot.PARALLEL_CALLBACK_BINDINGS


def test_parallel_session_runtime_issue_detects_shell_pane(monkeypatch, tmp_path: Path):
    """并行 tmux 虽存在，但 pane 当前进程仍是 shell 时，应视为未就绪/已失活。"""

    workspace_root = tmp_path / "workspace"
    workspace_root.mkdir(parents=True, exist_ok=True)
    pointer_file = tmp_path / "pointer.txt"
    pointer_file.write_text("placeholder", encoding="utf-8")
    session = SimpleNamespace(
        task_id="TASK_0115",
        status="running",
        tmux_session="vibe-par-hyphamall-task_0115",
        pointer_file=str(pointer_file),
        workspace_root=str(workspace_root),
    )

    monkeypatch.setattr(bot, "_parallel_tmux_session_exists", lambda _session_name: True)
    monkeypatch.setattr(bot, "_get_tmux_pane_current_command", lambda _session_name: "zsh", raising=False)

    issue = bot._parallel_session_runtime_issue(session)

    assert issue is not None
    assert "zsh" in issue


def test_delete_parallel_session_workspace_allows_cleanup_for_closed_session(monkeypatch, tmp_path: Path):
    """删除并行目录时应允许清理 closed/stale 会话，而不是只能清理 active 会话。"""

    task_id = "TASK_0115"
    runtime_root = tmp_path / task_id
    workspace_root = runtime_root / "workspace"
    workspace_root.mkdir(parents=True, exist_ok=True)
    session = SimpleNamespace(
        task_id=task_id,
        status="closed",
        tmux_session="vibe-par-hyphamall-task_0115",
        pointer_file=str(runtime_root / "_runtime" / "current_session.txt"),
        workspace_root=str(workspace_root),
    )

    deleted_calls: list[dict] = []
    updates: list[tuple[str, dict]] = []

    async def fake_get_session(_task_id: str):
        return session

    async def fake_update_status(_task_id: str, **kwargs):
        updates.append((_task_id, kwargs))

    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "get_session", fake_get_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "update_status", fake_update_status)
    monkeypatch.setattr(
        bot,
        "delete_parallel_workspace",
        lambda **kwargs: deleted_calls.append(kwargs),
    )
    monkeypatch.setattr(bot, "_parallel_runtime_root", lambda _task_id: runtime_root)
    bot.PARALLEL_TASK_SESSION_MAP[task_id] = "session-key"
    bot.PARALLEL_SESSION_CONTEXTS["session-key"] = _parallel_context(task_id)
    bot.PARALLEL_CALLBACK_BINDINGS = {"deadbeef": SimpleNamespace(task_id=task_id)}

    asyncio.run(bot._delete_parallel_session_workspace(task_id))

    assert deleted_calls, "应调用删除目录"
    assert updates and updates[-1][1]["status"] == "deleted"
    assert task_id not in bot.PARALLEL_TASK_SESSION_MAP
    assert "session-key" not in bot.PARALLEL_SESSION_CONTEXTS


def test_delete_parallel_session_workspace_cleans_session_scoped_runtime_state(monkeypatch, tmp_path: Path):
    """删除并行目录时，应同步清理会话级缓存状态，避免残留到 vibego 重启前。"""

    task_id = "TASK_0117"
    session_key = str(tmp_path / "runtime" / "session.jsonl")
    runtime_root = tmp_path / task_id
    workspace_root = runtime_root / "workspace"
    workspace_root.mkdir(parents=True, exist_ok=True)
    session = SimpleNamespace(
        task_id=task_id,
        status="committed",
        tmux_session="vibe-par-hyphamall-task_0117",
        pointer_file=str(runtime_root / "_runtime" / "current_session.txt"),
        workspace_root=str(workspace_root),
    )

    async def fake_get_session(_task_id: str):
        return session

    async def fake_update_status(_task_id: str, **_kwargs):
        return None

    async def fake_cleanup(path: Path, *, scope: str, owner_key: str):
        return None

    bot.PARALLEL_TASK_SESSION_MAP[task_id] = session_key
    bot.PARALLEL_SESSION_CONTEXTS[session_key] = _parallel_context(task_id)
    bot.PARALLEL_SESSION_TASK_BINDINGS[session_key] = task_id
    bot.PARALLEL_CALLBACK_BINDINGS = {
        "deadbeef": SimpleNamespace(task_id=task_id, session_key=session_key),
    }
    bot.SESSION_OFFSETS[session_key] = 12
    bot.PENDING_SUMMARIES[session_key] = bot.PendingSummary(
        task_id=task_id,
        request_id="req-1",
        actor="Tester",
        session_key=session_key,
        session_path=Path(session_key),
        created_at=time.monotonic(),
    )
    bot.CHAT_LAST_MESSAGE[1] = {session_key: "历史消息"}
    bot.CHAT_DELIVERED_HASHES[1] = {session_key: {"hash-1"}}
    bot.CHAT_DELIVERED_OFFSETS[1] = {session_key: {1}}
    bot.CHAT_PARALLEL_REPLY_TARGETS[1] = {
        "task_id": task_id,
        "dispatch_context": _parallel_context(task_id),
        "token": "deadbeef",
        "expires_at": time.time() + 600,
    }

    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "get_session", fake_get_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "update_status", fake_update_status)
    monkeypatch.setattr(bot, "_cleanup_codex_trusted_project_path", fake_cleanup)
    monkeypatch.setattr(bot, "delete_parallel_workspace", lambda **_kwargs: None)
    monkeypatch.setattr(bot, "_parallel_runtime_root", lambda _task_id: runtime_root)

    asyncio.run(bot._delete_parallel_session_workspace(task_id))

    assert session_key not in bot.SESSION_OFFSETS
    assert session_key not in bot.PENDING_SUMMARIES
    assert bot.CHAT_LAST_MESSAGE.get(1, {}).get(session_key) is None
    assert bot.CHAT_DELIVERED_HASHES.get(1, {}).get(session_key) is None
    assert bot.CHAT_DELIVERED_OFFSETS.get(1, {}).get(session_key) is None
    assert 1 not in bot.CHAT_PARALLEL_REPLY_TARGETS
    assert "deadbeef" not in bot.PARALLEL_CALLBACK_BINDINGS
    assert session_key not in bot.PARALLEL_SESSION_TASK_BINDINGS


def test_delete_parallel_session_workspace_cleans_parallel_workspace_trust(monkeypatch, tmp_path: Path):
    """删除并行目录时，应同步清理该 workspace 的 Codex trusted 条目。"""

    task_id = "TASK_0116"
    runtime_root = tmp_path / task_id
    workspace_root = runtime_root / "workspace"
    workspace_root.mkdir(parents=True, exist_ok=True)
    session = SimpleNamespace(
        task_id=task_id,
        status="closed",
        tmux_session="vibe-par-hyphamall-task_0116",
        pointer_file=str(runtime_root / "_runtime" / "current_session.txt"),
        workspace_root=str(workspace_root),
    )

    cleanup_calls: list[tuple[str, str, str]] = []

    async def fake_get_session(_task_id: str):
        return session

    async def fake_update_status(_task_id: str, **_kwargs):
        return None

    async def fake_cleanup(path: Path, *, scope: str, owner_key: str):
        cleanup_calls.append((str(path), scope, owner_key))

    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "get_session", fake_get_session)
    monkeypatch.setattr(bot.PARALLEL_SESSION_STORE, "update_status", fake_update_status)
    monkeypatch.setattr(bot, "_cleanup_codex_trusted_project_path", fake_cleanup)
    monkeypatch.setattr(bot, "delete_parallel_workspace", lambda **_kwargs: None)
    monkeypatch.setattr(bot, "_parallel_runtime_root", lambda _task_id: runtime_root)

    asyncio.run(bot._delete_parallel_session_workspace(task_id))

    assert cleanup_calls == [(str(workspace_root), "parallel_workspace", task_id)]
